using System;
using System.Text;
using Zcu.Mve.Core;
using Zcu.Mve.Numerics;
using System.ComponentModel;

namespace Zcu.Mve.Numerics
{
    /// <summary>
    /// A module that linearly transforms MveInt on the input. Coeficients of the transform are specified by the A and B properties, the result is computed as A*input+B.
    /// </summary>
    [ModuleInfo("Libor Va", "A module that linearly transforms MveInt on the input. Coeficients of the transform are specified by the A and B properties, the result is computed as A*input+B.", IconName = "LinearTransform.ico")]
    [PortInfo("Input", "Input MveInt value")]
    [PortInfo("Output", "Output Scalar value")]
    public class LinearTransform:Module
    {
        /// <summary>
        /// Default constructor, adds input and output ports.
        /// </summary>
        public LinearTransform()
        {
            AddInPort("Input", typeof(Integer));
            AddOutPort("Output", typeof(Scalar));
        }

        private bool paramsChanged = true;

        private double result;

        private double a = 1, b = 0;

        /// <summary>
        /// Specifies the multiplier in the linear transform
        /// </summary>
        [Description("Specifies the multiplier in the linear transform")]
        public double A
        {
            get { return (a); }
            set 
            { 
                a = value;
                paramsChanged = true;
            }
        }

        /// <summary>
        /// Specifies the absolute part of the linear transform
        /// </summary>
        [Description("Specifies the absolute part of the linear transform")]
        public double B
        {
            get { return (b); }
            set 
            { 
                b = value;
                paramsChanged = true;
            }
        }

        /// <summary>
        /// Performs the linear transformation.
        /// </summary>
        public override void Execute()
        {
            if (paramsChanged||(!IsAllDataSame()))
            {
                Integer input = (Integer)GetInput("Input");
                result = input.Value * a + b;
                SetOutput("Output", new Scalar(result));
                paramsChanged = false;
            }
            else
            {
                SetOutput("Output", new Scalar(result), true);
            }
        }
    }
}
