using System;
using Zcu.Mve.Core;
using System.ComponentModel;

namespace Zcu.Mve.Numerics
{
    /// <summary>
    /// A source module for Scalar
    /// </summary>
    [ModuleInfo("Libor Va", "A source module for Scalar", IconName = "ScalarSource.ico")]
    [PortInfo("Scalar", "Resulting Scalar")]
    public class ScalarSource : Module
    {
        private Random r = new Random();
        private Scalar output;
        private double val;
        private bool randomize = false;
        private int randomSeed = 0;
        private bool forceExecution = false;

        /// <summary>
        /// Scalar value that will be set as output. If randomized is set to true, Value is used as upper limit.
        /// </summary>        
        [Description("Scalar value that will be used as output of the module. If randomized is set to true, Value is used as upper limit.")]
        [Category("Data")]
        public double Value
        {
            get { return (val); }
            set { val = value; }
        }

        /// <summary>
        /// Generate random Scalar. 
        /// </summary>
        [Description("Generate random Scalar")]
        [Category("Random")]
        public bool Randomize
        {
            get { return randomize; }
            set { randomize = value; }
        }

        /// <summary>
        /// Random seed. 
        /// </summary>
        [Description("Random seed")]
        [Category("Random")]
        public int RandomSeed
        {
            get { return randomSeed; }
            set
            {
                randomSeed = value;
                if (randomSeed < 0) r = new Random();
                else r = new Random(randomSeed);
            }
        }

        /// <summary>
        /// Generate random Scalar. 
        /// </summary>
        [Description("Force execution when nothing changed.")]
        [Category("Execution")]
        public bool ForceExecution
        {
            get { return forceExecution; }
            set { forceExecution = value; }
        }



        /// <summary>
        /// Default constructor.
        /// </summary>
        public ScalarSource()
        {
            AddOutPort("Scalar", typeof(Scalar));
        }

        /// <summary>
        /// Checks wether value property was changed and sets the output accordingly.
        /// If value was not changed since the last run then dataSame is set to true.
        /// </summary>
        public override void Execute()
        {
            if (IsAllUserSettingsSame() && !forceExecution)
            {
                this.WriteDebug("Skip execution", "nothing changed");
                this.ProgressInfo(1);
                this.SetOutput("Scalar", output, true);
            }
            else
            {
                if (randomize)
                {
                    output = new Scalar(r.Next());
                    this.ProgressInfo(1);
                    this.SetOutput("Scalar", output, false);
                }
                else
                {
                    output = new Scalar(this.val);
                    this.ProgressInfo(1);
                    this.SetOutput("Scalar", output, false);
                }
            }
        }

    }
}
