using System;
using Zcu.Mve.Core;
using Zcu.Mve.Numerics;

namespace Zcu.Mve.Strings
{
	/// <summary>
	/// Module that composes a filename form a prefix, number and a suffix. Leading zeroes may be optionally added for the number to be of given length
	/// </summary>
	[ModuleInfo("Libor Va","Module that composes a filename form a prefix, number and a suffix. Leading zeroes may be optionally added for the number to be of given length")]
	[PortInfo("Prefix","An MveString port that determines the first part of the resulting filename.")]
	[PortInfo("Suffix","An MveString port that determines the last part of the resulting filename.")]
	[PortInfo("Number","An MveInt port that determines the number that will appear in the middle of the resulting filename.")]
	[PortInfo("Filename","An MveString port that exposes the composed filename")]
	public class FileNameComposer:Module
	{        
		private bool paramsChanged = true;
		private bool addLeadingZeros = false;
		private int numberLength = 3;

		[System.ComponentModel.Description("Sets whether or not leading zeros should be added to the beginning of the number")]
		public bool AddLeadingZeros
		{
			get{return(addLeadingZeros);}
			set
			{
				addLeadingZeros = value;
				paramsChanged = true;
			}
		}

		[System.ComponentModel.Description("Sets the total length of the number part of the filename. If AddLeadingZeros is set to true, then leading zeros will be added to each number so that the set length is matched, otherwise has no effect.")]
		public int NumberLength
		{
			get{return(numberLength);}
			set
			{
				if (value<1)
				{
					throw new MveException(this, "Number length must be greater or equal to one for the FileNameComposer module!");
				}
				else
				{
					numberLength = value;
					paramsChanged = true;
				}
			}
		}

		MveString output; // output for the case that nothing changes

		public FileNameComposer()
		{
			AddInPort("Prefix",typeof(MveString));
			AddInPort("Number",typeof(Integer));
			AddInPort("Suffix",typeof(MveString));
			AddOutPort("Filename",typeof(MveString));
		}

		public override void Execute()
		{
			if (paramsChanged||(!IsAllDataSame()))
			{
				MveString prefix = (MveString)GetInput("Prefix");
				MveString suffix = (MveString)GetInput("Suffix");
				Integer number = (Integer)GetInput("Number");

				string result = "";
				result+=prefix.Value;
				string numberString = number.Value.ToString();
				if (addLeadingZeros)
				{
					int len = numberString.Length;
					string zeros = "";
					for (int i = 0;i<(numberLength-len);i++)
						zeros+="0";	
					result += zeros;
				}
				result += numberString;
				result += suffix.Value;

				output = new MveString(result);
				SetOutput("Filename",output,false);
				paramsChanged = false;
			}
			else
			{
				SetOutput("Filename",output,true);
			}
		}
	}
}
