using System;
using Zcu.Mve.Core;
using System.ComponentModel;

namespace Zcu.Mve.Strings
{
	/// <summary>
	/// A source of constant string. Use the Value property to set the string value.
	/// </summary>
	[ModuleInfo("Libor Va","A source of constant string. Use the Value property to set the string value.")]
	[PortInfo("String","Output MveString.")]
	public class StringSource:Module
	{
		private string str;	// internal string value
		private bool paramsChanged = true; // flag that determines wether a new value should be created
		MveString output; // output MveString that will be passed if no properties are changed

		[Description("String value that will appear as output. Should not contain the ]]> sequence as it cannot be exported to XML.")]
		public string Value
		{
			get{return(str);}
			set
			{
				str = value;
				paramsChanged = true;
			}
		}

		public StringSource()
		{
			AddOutPort("String",typeof(MveString));
		}

		public override void Execute()
		{
			if (paramsChanged)
			{
				output = new MveString(str);
				SetOutput("String",output,false);
				paramsChanged = false;
			}
			else
			{
				SetOutput("String",output,true);
			}
		}
	}
}
