using System;
using Zcu.Mve.Core;
using Zcu.Mve.Numerics;
using Zcu.Mve.Visualization;

namespace Zcu.Mve.Testing
{
	/// <summary>
	/// 
	/// </summary>
	public class ElevationFilter : Module
	{
    internal int axis = 0;

		public ElevationFilter()
		{
      AddInPort("Mesh", typeof(Mesh));
      AddOutPort("Mesh", typeof(Mesh));
    }
	
    public override void Execute()
    {
      Mesh inMesh = (Mesh) GetInput("Mesh");

      if (inMesh.Points.ElemType != typeof(Point3D))
        throw new MveException(this, "Only Point3D are allowed in ElevationFilter.");

    Mesh outMesh = (Mesh)inMesh.ShallowCopy();

      UniformDataArray values = new UniformDataArray(typeof(Scalar), inMesh.Points.Count);

      double min = inMesh.BoundingBox.MinPoint[axis];
      double max = inMesh.BoundingBox.MaxPoint[axis];
      double dif = max - min;
      
      for (int i = 0; i < inMesh.Points.Count; i++)
      {
        values[i] = new Scalar((((Point3D)inMesh.Points[i])[axis] - min) / dif);
      }

      outMesh.AddPointAttrs(typeof(Scalar).Name, values);

      SetOutput("Mesh", outMesh);
    }

    /// <summary>
    /// Called when something is about to connect to an input port of this module.
    /// This operation can be denied by seeting allowed to false.
    /// </summary>
    /// <param name="source"></param>
    /// <param name="portFrom"></param>
    /// <param name="portTo"></param>
    /// <param name="allowed"></param>
    void Connect(object source, OutPort portFrom, InPort portTo, out bool allowed) 
    {
      Console.WriteLine("Someone from module " + portFrom.Owner.Name + " is trying to connect on port " + portTo.Name + " in module " + this.Name + ".");
      allowed = true;
    }
  
    public override ModuleSetup InvokeSetup()
    {
      return new ElevationSetup(this);
    }
  }
}
