using System;
using System.Collections.Generic;
using System.Text;

using Zcu.Mve.Core;
using Zcu.Mve.Visualization;
using System.ComponentModel;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using System.Xml;
using System.Diagnostics;

namespace Zcu.Mve.UnstrGridRendering
{

    [ModuleInfo("Vojetch Hladik", " collects meshes into array and gives them frame number")]
    [PortInfo("sceneObejcArray", "Array containig all frames.")]
    [PortInfo("meshIndex", "Frame index to load.")]
    [PortInfo("sceneObject", "Sigle frame to render.")]
    class AnimationComposer: Module
    {
        #region fields
        private int meshCount;
        private UniformDataArray dataArray;
        #endregion

        #region properties
        [Category("Animation")]
        [Description("number of meshes in animation")]
        public int MeshCount
        {
            get { return meshCount; }
            set { meshCount = value; }
        }

        #endregion

        /// <summary>
        /// creare AnimationComposer
        /// </summary>
        public AnimationComposer()
        {
            meshCount = 5;

            AddOutPort("sceneObejcArray", typeof(Mve.Visualization.UniformDataArray),true);
            AddOutPort("meshIndex", typeof(Mve.Numerics.Integer), false);
            AddInPort("sceneObject", typeof(SceneObject),true,false);

            dataArray = null;
            this.SimulationStarted += new SimulationStartedEventHandler(AnimationComposer_SimulationStarted);

        }

        /// <summary>
        /// clear anitation array
        /// </summary>
        void AnimationComposer_SimulationStarted(object source)
        {
            dataArray = null;
        }

        public override void Execute()
        {

            if (dataArray==null)
            {
                dataArray = new UniformDataArray(typeof(SceneObject), meshCount);

                for(Mve.Numerics.Integer i = 0; i<meshCount; i++)
                {
                    SetOutput("meshIndex", (Mve.Numerics.Integer)(i));
                    UpdateInput("sceneObject");
                    dataArray[i] = GetInput("sceneObject").DeepCopy();
       
                    ((SceneObject)dataArray[i]).SetFrame(i);
                }
                SetOutput("sceneObejcArray", dataArray,false);
            }
            else SetOutput("sceneObejcArray", dataArray, true);

        }

        /// <summary>
        /// Writes Xml configuration of current module.
        /// </summary>
        /// <param name="config">XmlElement from map config file which belongs to current module.</param>
        public override void WriteConfig(XmlElement config)
        {
            XmlElement xmlProperties = config.OwnerDocument.CreateElement("Properties");

            XmlElement xmlFilename = config.OwnerDocument.CreateElement("meshCount");
            xmlFilename.SetAttribute("type", meshCount.GetType().FullName);
            xmlFilename.InnerText = meshCount.ToString();
            xmlProperties.AppendChild(xmlFilename);

            config.AppendChild(xmlProperties);
        }

        /// <summary>
        /// Reads Xml configuration of current module.
        /// </summary>
        /// <param name="config">XmlElement from map config file which belongs to current module.</param>
        public override void ReadConfig(XmlElement config)
        {
            foreach (XmlElement xeConfig in config.ChildNodes)
            {
                if (xeConfig.Name == "Properties")
                {
                    foreach (XmlElement xeProperties in xeConfig.ChildNodes)
                    {
                        switch (xeProperties.Name)
                        {
                            case "meshCount":
                                meshCount = FromString.GetInt(xeProperties.InnerText,-1);
                                break;
                            default:
                                Debug.WriteLine("Unreaded value:" + xeProperties.Name + " - " + xeProperties.InnerText);
                                break;
                        }
                    }
                }
            }
        }
    }
}
