using System;
using System.Drawing;

using Microsoft.DirectX.Direct3D;

namespace Zcu.Mve.UnstrGridRendering.Legend
{
    /// <summary>
    /// Base abstract class used for legend visualization. Legend is draw onto screen, no transformation is used.
    /// </summary>
    public abstract class BaseLegend : Mve.Core.DataObject
    {
        protected Bitmap _bitmap;
        protected Rectangle _rect;
        protected Graphics _bmpGraphics;

        #region Properties

        public Point LegendLocation
        {
            get { return _rect.Location; }
            set { _rect.Location = value; }
        }
        
        public Size LegendSize
        {
            get { return _rect.Size; }
            set { _rect.Size = value; }
        }

        public Bitmap Bitmap
        {
            get { return this._bitmap; }
        }

        public Rectangle Rectangle
        {
            get { return this._rect; }
        }

        #endregion

        /// <summary>
        /// Contructor
        /// </summary>
        /// <param name="rect"></param>
        public BaseLegend(Rectangle rect)
        {
            _rect = rect;

            _bitmap = new Bitmap(_rect.Width, _rect.Height, System.Drawing.Imaging.PixelFormat.Format32bppArgb);

            _bmpGraphics = Graphics.FromImage(_bitmap);
        }

        /// <summary>
        /// Draw lened context onto screen.
        /// </summary>
        public void Draw()
        {
            if (_bmpGraphics != null) Draw(_bmpGraphics);
        }

        public void Dispose()
        {
            if (_bmpGraphics != null) _bmpGraphics.Dispose();
            if (_bitmap != null) _bitmap.Dispose();
        }

        /// <summary>
        /// Abstract method used for context legend creation. Graphics define methods for 
        /// drawing primitives, fonts etc.
        /// </summary>
        /// <param name="g"></param>
        protected abstract void Draw(Graphics g);

        #region DataObject

        public override bool CheckConsistence()
        {
            return true;
        }

        public override Zcu.Mve.Core.IDataObject DeepCopy()
        {
            throw new NotImplementedException();
        }

        public override void ReadData(System.Xml.XmlTextReader xmlTextReader)
        {
            
        }

        public override void WriteData(System.Xml.XmlTextWriter xmlTextWriter)
        {

        }

        #endregion
    }
}
