using System;
using System.Xml;
using System.Windows.Forms;

using Microsoft.DirectX;

using Zcu.Mve.Core;

namespace Zcu.Mve.UnstrGridRendering
{
    /// <summary>
    /// Class used for picking scene elements.
    /// </summary>
    public class BasePicker : Zcu.Mve.Core.DataObject
    {
        public event KeyEventHandler KeyUp;
        public event KeyEventHandler KeyDown;

        public event MouseEventHandler MouseUp;
        public event MouseEventHandler MouseDown;
        public event MouseEventHandler MouseMove;

        private Camera _camera;
        private int _mX, _mY;
        private int _w, _h;
        private Mve.Numerics.Vector3D _sLine;

        #region Properties

        public System.Drawing.Size WndSize
        {
            set { _w = value.Width; _h = value.Height; }
        }

        public Mve.Numerics.Vector3D SelectionLine
        {
            get 
            {
                ComputeSelectionLine();

                return _sLine; 
            }
        }

        #endregion

        /// <summary>
        /// Constructor
        /// </summary>
        public BasePicker(System.Drawing.Size wndSize, Camera camera)
        {
            _camera = camera;

            _w = wndSize.Width;
            _h = wndSize.Height;
        }

        public void ComputeSelectionLine()
        {
            Matrix p = new Matrix();

            p.M11 = _camera.ProjectionTransform[0, 0];
            p.M12 = _camera.ProjectionTransform[0, 1];
            p.M13 = _camera.ProjectionTransform[0, 2];
            p.M14 = _camera.ProjectionTransform[0, 3];

            p.M21 = _camera.ProjectionTransform[1, 0];
            p.M22 = _camera.ProjectionTransform[1, 1];
            p.M23 = _camera.ProjectionTransform[1, 2];
            p.M24 = _camera.ProjectionTransform[1, 3];

            p.M31 = _camera.ProjectionTransform[2, 0];
            p.M32 = _camera.ProjectionTransform[2, 1];
            p.M33 = _camera.ProjectionTransform[2, 2];
            p.M34 = _camera.ProjectionTransform[2, 3];

            p.M41 = _camera.ProjectionTransform[3, 0];
            p.M42 = _camera.ProjectionTransform[3, 1];
            p.M43 = _camera.ProjectionTransform[3, 2];
            p.M44 = _camera.ProjectionTransform[3, 3];

            p.Invert();

            float x = (float)_mX / (float)_w;
            float y = (float)_mY / (float)_h;

            x = x * 2.0f - 1.0f;
            y = y * 2.0f - 1.0f;

            Vector4 v = new Vector4(x, y, 1.0f, 0.0f);

            Matrix m = Matrix.LookAtLH(
                new Vector3((float)_camera.Position.X, (float)_camera.Position.Y, (float)_camera.Position.Z),
                new Vector3((float)_camera.Target.X, (float)_camera.Target.Y, (float)_camera.Target.Z),
                new Vector3((float)_camera.UpVector.X, (float)_camera.UpVector.Y, (float)_camera.UpVector.Z));

            v.Transform(m);

            v.Transform(p);

            _sLine = new Zcu.Mve.Numerics.Vector3D(v.X, v.Y, v.Z);
        }

        public void OnKeyUp(KeyEventArgs e) 
        {
            if (KeyUp != null) KeyUp(this, e);
        }

        public void OnKeyDown(KeyEventArgs e) 
        {
            if (KeyDown != null) KeyDown(this, e);
        }

        public void OnMouseMove(MouseEventArgs e) 
        {
            if (MouseMove != null) MouseMove(this, e);
        }

        public void OnMouseDown(MouseEventArgs e)
        {
            _mX = e.X;
            _mY = e.Y;

            if (MouseDown != null) MouseDown(this, e);
        }

        public void OnMouseUp(MouseEventArgs e)
        {
            if (MouseUp != null) MouseUp(this, e);
        }

        #region DataObject

        public override string ToString()
        {
            return "BasePicker";
        }

        public override bool CheckConsistence()
        {
            return true;
        }

        public override Zcu.Mve.Core.IDataObject DeepCopy()
        {
            BasePicker bp = new BasePicker(new System.Drawing.Size(_w, _h), _camera);

            return bp;
        }

        public override void ReadData(System.Xml.XmlTextReader xmlTextReader)
        {

        }

        public override void WriteData(System.Xml.XmlTextWriter xmlTextWriter)
        {

        }

        #endregion
    }
}
