using System;
using System.Collections.Generic;
using System.Text;
using Zcu.Mve.Core;
using Zcu.Mve.Visualization;
using Zcu.Mve.Numerics;
using System.ComponentModel;
using System.IO;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;

using System.Xml;
using System.Diagnostics;
using System.Drawing.Design;
using System.Globalization;
using System.Threading;

namespace Zcu.Mve.UnstrGridRendering
{
    /// <summary>
    /// class contains methods to compute color table
    /// </summary>
    class ColorMappingFunction : DataObject
    {

        private FunctionEnum functions;
        public virtual FunctionEnum Functions
        {
            get { return functions; }
            set { functions = value; }
        }


        public ColorMappingFunction()
        {
            functions = FunctionEnum.Grey;
        }

        /// <summary>
        /// convert byte value to color 
        /// </summary>
        /// <param name="value">value from 0 to 255</param>
        /// <returns>RGB</returns>
        public virtual ColorRGBA GetColor(byte value)
        {
            return GetColor(value/256.0f);
        }

        /// <summary>
        /// convert int value to color 
        /// </summary>
        /// <param name="value">value from 0 to 255</param>
        /// <returns>RGB</returns>
        public virtual ColorRGBA GetColor(int value)
        {
            return GetColor(value / 256.0f);
        }

        /// <summary>
        /// convert float value to color
        /// </summary>
        /// <param name="value">value from 0 to 1</param>
        /// <returns>RGB</returns>
        public virtual ColorRGBA GetColor(float value)
        {
            switch (functions)
            {
                case FunctionEnum.Grey:
                    {
                        return GreyColor(value);
                    }
                case FunctionEnum.GreyInverse:
                    {
                        return GreyColorInverse(value);
                    }
                case FunctionEnum.BlueToRed:
                    {
                        return BlueToRed(value);
                    }
                default:
                    {
                        return GreyColor(value);
                    }
            }
        }


        private ColorRGBA GreyColor(float value)
        {
            value *= 255;
            value = Math.Min(255, value);
            value = Math.Max(0, value);

            ColorRGBA  c = new ColorRGBA();
            byte byteValue = (byte)(value );

            c.Red = byteValue;
            c.Green = byteValue;
            c.Blue = byteValue;
            c.Alpha = 255;

            return c;
        }

        private ColorRGBA GreyColorInverse(float value)
        {
            value *= 255;
            value = Math.Min(255,  value);
            value = Math.Max(0,  value);

            ColorRGBA c = new ColorRGBA();
            byte byteValue = (byte)(255 - value);

            c.Red = byteValue;
            c.Green = byteValue;
            c.Blue = byteValue;
            c.Alpha = 255;

            return c;
        }

        private ColorRGBA BlueToRed(float value)
        {
            return hsvtorgb(360 - (60 + 300 * (value)), (float)(Math.Sqrt(value)), (float)(Math.Sqrt(value)));
        }



        private ColorRGBA hsvtorgb(float h, float s, float v)
        {
            ColorRGBA color = new ColorRGBA(); ;
            color.Alpha = 255;

            float hi =  ((int)(h / 60)) % 60;
            float f = h / 60.0f - hi;
            float p = v * (1 - s);
            float q = v * (1 - f * s);
            float t = v * (1 - (1 - f) * s);

            if (hi == 0)
            {
                color.Red = (byte)(255 * v);
                color.Green = (byte)(255 * t);
                color.Blue = (byte)(255 * p);
            }
            else if (hi == 1)
            {
                color.Red = (byte)(255 * q);
                color.Green = (byte)(255 * v);
                color.Blue = (byte)(255 * p);
            }
            else if (hi == 2)
            {
                color.Red = (byte)(255 * p);
                color.Green = (byte)(255 * v);
                color.Blue = (byte)(255 * t);
            }
            else if (hi == 3)
            {
                color.Red = (byte)(255 * p);
                color.Green = (byte)(255 * q);
                color.Blue = (byte)(255 * v);
            }
            else if (hi == 4)
            {
                color.Red = (byte)(255 * t);
                color.Green = (byte)(255 * p);
                color.Blue = (byte)(255 * v);
            }
            else
            {
                color.Red = (byte)(255 * v);
                color.Green = (byte)(255 * p);
                color.Blue = (byte)(255 * q);
            }

            return color;

        }

        public override bool CheckConsistence()
        {
            return true;
        }

        public override IDataObject DeepCopy()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public override void ReadData(XmlTextReader xmlTextReader)
        {
            
        }

        public override void WriteData(XmlTextWriter xmlTextWriter)
        {

        }
    }
}
