using System;
using System.Collections.Generic;
using System.Text;
using Zcu.Mve.Core;
using Zcu.Mve.Visualization;
using Zcu.Mve.Numerics;
using System.ComponentModel;
using System.IO;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.Windows.Forms;

using System.Xml;
using System.Diagnostics;
using System.Drawing.Design;
using System.Globalization;
using System.Threading;

namespace Zcu.Mve.UnstrGridRendering
{
    public enum FunctionEnum { Grey = 1, GreyInverse = 2, BlueToRed = 3 };

    [ModuleInfo("Vojtech Hladik", "module which encapsulates color mapping function")]
    [PortInfo("ColorFunction", "provides color table")]
    class ColorMappingFunctionModule: Module
    {
        private ColorMappingFunction fcDelegate;

        [Category("Color settings"),
        Description("")]
        public FunctionEnum MappingFunctions
        {
            get { return fcDelegate.Functions; }
            set { fcDelegate.Functions = value;}
        }

        public ColorMappingFunctionModule()
        {
            AddOutPort("ColorFunction", typeof(ColorMappingFunction));
            fcDelegate = new ColorMappingFunction();
        }

        public override void Execute()
        {
            SetOutput("ColorFunction", fcDelegate);
        }

        /// <summary>
        /// Writes Xml configuration of current module.
        /// </summary>
        /// <param name="config">XmlElement from map config file which belongs to current module.</param>
        public override void WriteConfig(XmlElement config)
        {
            XmlElement xmlProperties = config.OwnerDocument.CreateElement("Properties");

            XmlElement xmlFunction = config.OwnerDocument.CreateElement("functions");
            xmlFunction.SetAttribute("type", fcDelegate.Functions.GetType().FullName);
            xmlFunction.InnerText = fcDelegate.Functions.ToString();
            xmlProperties.AppendChild(xmlFunction);

            config.AppendChild(xmlProperties);
        }

        /// <summary>
        /// Reads Xml configuration of current module.
        /// </summary>
        /// <param name="config">XmlElement from map config file which belongs to current module.</param>
        public override void ReadConfig(System.Xml.XmlElement config)
        {
            foreach (XmlElement xeConfig in config.ChildNodes)
            {
                if (xeConfig.Name == "Properties")
                {
                    foreach (XmlElement xeProperties in xeConfig.ChildNodes)
                    {
                        switch (xeProperties.Name)
                        {
                            case "functions":
                                fcDelegate.Functions = (FunctionEnum)Enum.Parse(typeof(FunctionEnum), xeProperties.InnerText);
                                break;
                            default:
                                Debug.WriteLine("Unreaded value:" + xeProperties.Name + " - " + xeProperties.InnerText);
                                break;
                        }
                    }
                }
            }
        }
    }
}
