using System;
using System.Collections.Generic;
using System.Text;

namespace Zcu.Mve.UnstrGridRendering
{
  /// <summary>
  /// Helper class of D3DUTRenderer. It is used to count displayed frames per second.
  /// </summary>
  public class FPSCounter
  {
    private int lastTickCount;
    private int fpsTmp;
    private int fps;

    /// <summary>
    /// Default constructor. Resets counter.
    /// </summary>
    public FPSCounter()
    {
      Reset();
    }

    /// <summary>
    /// Reset the counter.
    /// </summary>
    public void Reset()
    {
      lastTickCount = System.Environment.TickCount;
      fps = 0;
      fpsTmp = 0;
    }

    /// <summary>
    /// Updates counter. This should be performed before every frame rendering.
    /// </summary>
    public void Update()
    {
      fpsTmp++;

      if (System.Environment.TickCount - lastTickCount > 1000)
      {
        lastTickCount = System.Environment.TickCount;
        fps = fpsTmp;
        fpsTmp = 0;
      }
    }

    /// <summary>
    /// Get actual FPS.
    /// </summary>
    /// <returns>Actual framerate.</returns>
    public int GetFPS()
    {
      return fps;
    }
  }
}
