using System;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

namespace Zcu.Mve.UnstrGridRendering
{
    /// <summary>
    ///
    /// </summary>
    public class FontWrapper : IDisposable
    {
        private Font _font;

        private bool _enable;

        #region Properties

        public bool Enable
        {
            get { return _enable; }
            set { _enable = value; }
        }

        #endregion

        /// <summary>
        /// Default constructor. Just takes reference to runtime.
        /// </summary>
        /// <param name="runtime"></param>
        public FontWrapper(Device device)
        {
            CreateFont(device);

            _enable = true;

            device.DeviceLost += new EventHandler(device_DeviceLost);
            device.DeviceReset += new EventHandler(device_DeviceReset);
        }

        private void device_DeviceReset(object sender, EventArgs e)
        {
            if ((_font != null) && !_font.Disposed) _font.OnResetDevice();
        }

        private void device_DeviceLost(object sender, EventArgs e)
        {
            if ((_font != null) && !_font.Disposed) _font.OnLostDevice();
        }

        /// <summary>
        /// Creates Font.
        /// </summary>
        /// <param name="device">Reference to rendering device.</param>
        protected void CreateFont(Device device)
        {
            if (device != null)
            {
                _font = new Font(device, 8, 0, FontWeight.Normal, 0, false, CharacterSet.Default,
                  Precision.Default, FontQuality.Default, PitchAndFamily.DefaultPitch, "FixedSys");
            }
        }

        public void Draw(string text)
        {
            if (_enable) _font.DrawText(null, text, 5, 5, System.Drawing.Color.Gray);
        }

        #region IDisposable Members

        public void Dispose()
        {
            if (_font != null && !_font.Disposed) _font.Dispose();

            _font = null;
        }

        void IDisposable.Dispose()
        {
            if (_font != null && !_font.Disposed) _font.Dispose();
        }

        #endregion
    }
}
