using System;
using System.Collections.Generic;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

namespace Zcu.Mve.UnstrGridRendering.Legend
{
    /// <summary>
    /// Legend renderer, legends are draw in batches using Sprite. Class is dependent on Device. Device events
    /// are handled internaly by class, so user cannot handle this events.
    /// </summary>
    public class LegendRenderer : IDisposable
    {
        private Sprite _sprite;
        private Device _device;
        private LegendComposer _composer;

        #region Composer

        public LegendComposer Composer
        {
            get { return _composer; }
        }

        #endregion

        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="device"></param>
        public LegendRenderer(Device device)
        {
            _device = device;

            _composer = new LegendComposer(_device);

            _sprite = new Sprite(_device);

            device.DeviceReset  += new EventHandler(device_DeviceReset);
            device.DeviceLost   += new EventHandler(device_DeviceLost);
        }

        /// <summary>
        /// Rendering the legend batch.
        /// </summary>
        public void Render()
        {
            _device.BeginScene();

            _sprite.Begin(SpriteFlags.AlphaBlend | SpriteFlags.DoNotSaveState | SpriteFlags.DoNotModifyRenderState);

            _composer.Render(_sprite);

            _sprite.End();

            // Sprite state modify RenderState ... dont know why !!!
            _device.RenderState.AlphaBlendEnable = false;
            _device.RenderState.AlphaTestEnable = false;

            _device.EndScene();
        }

        void device_DeviceReset(object sender, EventArgs e)
        {
            if (_sprite != null) _sprite.OnResetDevice();

            if (_composer != null) _composer.OnDeviceReset(_device);
        }

        void device_DeviceLost(object sender, EventArgs e)
        {
            if (_composer != null) _composer.OnDeviceLost(_device);

            if (_sprite != null && !_sprite.Disposed) _sprite.OnLostDevice();
        }

        #region IDisposable Members

        public void Dispose()
        {
            if (_sprite != null && !_sprite.Disposed) _sprite.Dispose();

            _sprite = null;

            _device = null;
        }

        void IDisposable.Dispose()
        {
            if (_sprite != null && !_sprite.Disposed) _sprite.Dispose();

            _sprite = null;

            _device = null;
        }

        #endregion
    }
}
