using System;
using System.Collections.Generic;
using System.Text;
using dx3d = Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;
using System.Diagnostics;
using System.Drawing;

namespace Zcu.Mve.UnstrGridRendering
{
  /// <summary>
  /// Class holds Material properties for D3D object representation.
  /// It also performs transition from Material to MaterialWrapper. 
  /// </summary>
  public class MaterialWrapper
  {
    private Material material;
    private dx3d.Direct3D.Material d3dutMaterial;
    private long lmtMaterial;

    /// <summary>
    /// Default constructor. Initializes current object and saves reference to parent SceneObject.
    /// </summary>
    public MaterialWrapper()
    {
      this.lmtMaterial = 0;
    }

    /// <summary>
    /// Update d3dutMaterial properties.
    /// </summary>
    /// <param name="material">Material.</param>
    /// <param name="materialEnable">Material enable scene objects property.</param>
    public void Update(Material material, bool materialEnable)
    {
      if ((material != null) && (lmtMaterial != material.ModificationTime))
      {
        this.material = material;

        if (materialEnable)
        {
          d3dutMaterial = new dx3d.Direct3D.Material();
          d3dutMaterial.Ambient = Color.FromArgb((int)material.Ambient.Alpha,
                                                       (int) material.Ambient.Red,
                                                       (int) material.Ambient.Green,
                                                       (int) material.Ambient.Blue);

          d3dutMaterial.Diffuse = Color.FromArgb((int)material.Diffuse.Alpha,
                                                       (int) material.Diffuse.Red,
                                                       (int) material.Diffuse.Green,
                                                       (int) material.Diffuse.Blue);

          d3dutMaterial.Emissive = Color.FromArgb((int)material.Emissive.Alpha,
                                                        (int) material.Emissive.Red,
                                                        (int) material.Emissive.Green,
                                                        (int) material.Emissive.Blue);

          d3dutMaterial.Specular = Color.FromArgb((int)material.Specular.Alpha,
                                                        (int) material.Specular.Red,
                                                        (int) material.Specular.Green,
                                                        (int) material.Specular.Blue);

          d3dutMaterial.SpecularSharpness = material.SpecularSharpness;

          lmtMaterial = material.ModificationTime;
        }
        else
        {
          d3dutMaterial = new dx3d.Direct3D.Material();
        }

        Debug.WriteLine(">> Material - update: " + lmtMaterial);
      }
    }

    /// <summary>
    /// This method should be called during rendering between device.BeginScene() and device.EndScene().
    /// It sets d3dutMaterial properties in device render state.
    /// </summary>
    /// <param name="device">Rendering device.</param>
    public void SetMaterial(Device device)
    {
      if (material == null)
      {
        device.RenderState.SpecularEnable = false;
      }
      else
      {
        device.RenderState.SpecularEnable = material.SpecularEnable;
      }

      device.Material = d3dutMaterial;
    }
  }
}
