using System;
using System.IO;
using System.Drawing;

namespace Zcu.Mve.UnstrGridRendering
{
    /// <summary>
    /// Define fullscreen monitor.
    /// </summary>
    public enum ScreenTarget { Primary, Secondary };

    /// <summary>
    /// Structure that holds renderer settings.
    /// </summary>
    public struct RendererParameters
    {
        private int saveTextureFPS;

        public int SavetextureFPS
        {
            get { return saveTextureFPS; }
            set { saveTextureFPS = value; }
        }

        private bool pickerEnabled;

        public bool PickerEnabled
        {
            get { return pickerEnabled; }
            set { pickerEnabled = value; }
        }

        private ScreenTarget screen;

        public ScreenTarget Screen
        {
            get { return screen; }
            set { screen = value; }
        }

        private StereoSceneRendererModule.Mode stereoMode;

        public StereoSceneRendererModule.Mode StereoMode
        {
            get { return stereoMode; }
            set { stereoMode = value; }
        }

        private String pathSaveto;

        /// <summary>
        /// where to save images, when animation is recorded
        /// </summary>
        public String PathSaveto
        {
            get { return pathSaveto; }
            set 
            {
                try
                {
                    DirectoryInfo di = new DirectoryInfo(value);

                    if (!di.Exists) { di.Create(); }

                    pathSaveto = value; 
                }

                catch (Exception)
                {
                    pathSaveto = string.Empty;
                }
            }
        }

        private float eyeParalax;
        /// <summary>
        /// stereo - eye distance
        /// </summary>
        public float EyeParalax
        {
            get { return eyeParalax; }
            set { eyeParalax = value; }
        }

        private int width;
        /// <summary>
        /// Output window gridSize. Width of renderer window.
        /// </summary>
        public int Width
        {
            get
            {
                return width;
            }
            set
            {
                width = value;
            }
        }

        private int height;
        /// <summary>
        /// Output window gridSize. Height of renderer window.
        /// </summary>
        public int Height
        {
            get
            {
                return height;
            }
            set
            {
                height = value;
            }
        }

        private string text;
        /// <summary>
        /// Caption of renderer window.
        /// </summary>
        public string Text
        {
            get
            {
                return text;
            }
            set
            {
                text = (value == string.Empty) ? "Renderer" : value;
            }
        }

        private Color backgroundColor;
        /// <summary>
        /// Color used as a background color.
        /// </summary>
        public Color BackgroundColor
        {
            get
            {
                return backgroundColor;
            }
            set
            {
                backgroundColor = value;
            }
        }

        private bool useInternalCamera;

        /// <summary>
        /// "Use internal camera settings (position, fov, projection, etc.)
        /// or get these values from external CameraSource module.
        /// </summary>
        public bool UseInternalCamera
        {
            get
            {
                return useInternalCamera;
            }
            set
            {
                useInternalCamera = value;
            }
        }

        private bool useInternalVirtualTrackball;
        /// <summary>
        /// Use internal navigation thorough the scene
        /// or manage movement of objects and scene by yourself using some kind of interactor.
        /// </summary>
        public bool UseInternalVirtualTrackball
        {
            get
            {
                return useInternalVirtualTrackball;
            }
            set
            {
                useInternalVirtualTrackball = value;
            }
        }

        /// <summary>
        /// Performs deep copy of current struct.
        /// </summary>
        /// <returns>Deep copy of D3DUTRendererParameters struct.</returns>
        public RendererParameters DeepCopy()
        {
            RendererParameters rendererParameters = new RendererParameters();

            rendererParameters.text = this.text;
            rendererParameters.width = this.width;
            rendererParameters.height = this.height;
            rendererParameters.screen = this.screen;
            rendererParameters.saveTextureFPS = this.saveTextureFPS;
            rendererParameters.pickerEnabled = this.pickerEnabled;
            rendererParameters.eyeParalax = this.eyeParalax;
            rendererParameters.backgroundColor = this.backgroundColor;
            rendererParameters.useInternalCamera = this.useInternalCamera;
            rendererParameters.useInternalVirtualTrackball = this.useInternalVirtualTrackball;

            return rendererParameters;
        }
    }
}
