using System;
using System.Collections.Generic;
using System.Text;
using Zcu.Mve.Core;
using Zcu.Mve.Visualization;
using System.ComponentModel;

namespace Zcu.Mve.UnstrGridRendering
{
  /// <summary>
  /// Test module. Generates unstructured grid with sample of all supported primitives. 
  /// </summary>
  [ModuleInfo("Jan Kaiser", "Test module. Generates unstructured grid with sample of all supported primitives.")]
  [PortInfo("outUnstrGrid", "Resulting unstructured grid.")]
  public class TestModulePrimitives : Module
  {
    private UnstrGrid unstructuredGrid;

    private bool update;
    /// <summary>
    /// Enable/disable unstructured grid update on execute. 
    /// </summary>
    [Description("Enable/disable unstructured grid update on execute.")]
    [DefaultValue(false)]
    public bool Update
    {
      get
      {
        return update;
      }
      set
      {
        update = value;
      }
    }

    private int lastTime;
    private int updateDelay;
    /// <summary>
    /// If update is enabled, this specifies the delay between updates.
    /// </summary>
    [Description("If update is enabled, this specifies the delay between updates.")]
    [DefaultValue(1000)]
    public int UpdateDelay
    {
      get
      {
        return updateDelay;
      }
      set
      {
        updateDelay = value;
      }
    }

    /// <summary>
    /// Constructor, initializes current module. 
    /// </summary>
    public TestModulePrimitives()
    {
      this.Update = false;
      this.UpdateDelay = 1000;

      AddOutPort("outUnstrGrid", typeof(UnstrGrid));
      
      UpdatePrimitives();
      lastTime = System.Environment.TickCount;
    }

    /// <summary>
    /// Exposes generated unstructured grid on out port.
    /// </summary>
    public override void Execute()
    {
      if (update)
      {
        if ((System.Environment.TickCount - lastTime) > updateDelay)
        {
          UpdatePrimitives();

          lastTime = System.Environment.TickCount;
        }
      }

      SetOutput("outUnstrGrid", unstructuredGrid);
    }

    private void UpdatePrimitives()
    {
      unstructuredGrid = new UnstrGrid();
      UniformDataArray points = new UniformDataArray(typeof(Point3D), 48);

      points[0] = new Point3D(-1, 0, 0);
      points[1] = new Point3D(-0.8f, 0.5f, 0);
      points[2] = new Point3D(-0.6f, 0, 0);
      points[3] = new Point3D(-0.4f, 0.5f, 0);
      points[4] = new Point3D(-0.2, 0, 0);
      points[5] = new Point3D(0, 0.5f, 0);
      points[6] = new Point3D(1, -0.5f, 0);
      points[7] = new Point3D(0.8f, 0.5f, 0);
      points[8] = new Point3D(0.6f, -0.5f, 0);

      points[9] = new Point3D(-0.8f, -0.5f, 0);
      points[10] = new Point3D(-0.4f, -0.5f, 0);
      points[11] = new Point3D(0, -0.5f, 0);

      points[12] = new Point3D(0, 0, 0);
      points[13] = new Point3D(0.25f, 0.5f, 0);
      points[14] = new Point3D(0.4f, 0.4f, 0);
      points[15] = new Point3D(0.5f, 0.0f, 0);
      points[16] = new Point3D(0.4f, -0.4f, 0);
      points[17] = new Point3D(0.25f, -0.5f, 0);

      points[19] = new Point3D(-0.75f, 0.7f, 1f);
      points[21] = new Point3D(-0.25f, 0.8f, 1f);
      points[20] = new Point3D(-0.6f, 0.95f, 0.25f);
      points[18] = new Point3D(-0.5f, 0.55f, 0.25f);

      points[22] = new Point3D(-0.9f, -0.9f, 0f);
      points[23] = new Point3D(-0.7f, -0.6f, 0f);
      points[24] = new Point3D(-0.5f, -0.8f, 0f);
      points[25] = new Point3D(-0.3f, -0.7f, 0f);
      points[26] = new Point3D(-0.1f, -0.75f, 0f);

      points[27] = new Point3D(0.1f, -0.9f, 0f);
      points[28] = new Point3D(0.3f, -0.6f, 0f);
      points[29] = new Point3D(0.5f, -0.8f, 0f);
      points[30] = new Point3D(0.7f, -0.7f, 0f);
      points[31] = new Point3D(0.9f, -0.75f, 0f);
      
      points[32] = new Point3D(0.1f, 0.9f, 0f);
      points[33] = new Point3D(0.1f, 0.6f, 0f);
      points[34] = new Point3D(0.4f, 0.6f, 0f);
      points[35] = new Point3D(0.4f, 0.9f, 0f);

      points[36] = new Point3D(0.6f, 0.9f, 0f);
      points[37] = new Point3D(0.6f, 0.6f, 0f);
      points[38] = new Point3D(0.9f, 0.6f, 0f);
      points[39] = new Point3D(0.9f, 0.9f, 0f);

      points[40] = new Point3D(0.2f, 0.8f, 0.5f);
      points[41] = new Point3D(0.2f, 0.7f, 0.5f);
      points[42] = new Point3D(0.3f, 0.7f, 0.5f);
      points[43] = new Point3D(0.3f, 0.8f, 0.5f);

      points[44] = new Point3D(0.7f, 0.8f, 0.5f);
      points[45] = new Point3D(0.7f, 0.7f, 0.5f);
      points[46] = new Point3D(0.8f, 0.7f, 0.5f);
      points[47] = new Point3D(0.8f, 0.8f, 0.5f);

      //points[48] = new Point3D(0.0f, 0.0f, 0.0f);
      //points[49] = new Point3D(1f, 0.0f, 0.0f);
      //points[50] = new Point3D(0.0f, 0.0f, 0.0f);
      //points[51] = new Point3D(0.0f, 1f, 0.0f);
      //points[52] = new Point3D(0.0f, 0.0f, 0.0f);
      //points[53] = new Point3D(0.0f, 0.0f, 1f);

      unstructuredGrid.Points = points;

      UniformDataArray pTexture = new UniformDataArray(typeof(TextureCoord), points.Count);

      for (int i = 0; i < points.Count; i++)
      {
        Point3D p = (Point3D) points[i];
        pTexture[i] = new TextureCoord((float) p.X, (float) p.Y, 0);
      }

      unstructuredGrid.AddPointAttrs("TextureUV", pTexture);

      UniformDataArray pColor = new UniformDataArray(typeof(ColorRGBA), points.Count);

      for (int i = 0; i < points.Count; i++)
      {
        Point3D p = (Point3D) points[i];
        pColor[i] = new ColorRGBA((float) (p.X + 1) / 2f, (float) (p.Y + 1) / 2f, (float) (p.Z + 1) / 2f, 1f);
      }

      //pColor[48] = new ColorRGBA(1f, 0f, 0f, 1f);
      //pColor[49] = new ColorRGBA(1f, 0f, 0f, 1f);

      //pColor[50] = new ColorRGBA(0f, 1f, 0f, 1f);
      //pColor[51] = new ColorRGBA(0f, 1f, 0f, 1f);

      //pColor[52] = new ColorRGBA(0f, 0f, 1f, 1f);
      //pColor[53] = new ColorRGBA(0f, 0f, 1f, 1f);

      unstructuredGrid.AddPointAttrs("ColorRGBA", pColor);

      UniformDataArray cells = new UniformDataArray(typeof(TriangleStrip), 2);

      cells[0] = new TriangleStrip(new int[] { 0, 1, 2, 3, 4, 5 });
      cells[1] = new TriangleStrip(new int[] { 0, 9, 2, 10, 4, 11 });

      unstructuredGrid.AddCells("TriangleStrip", cells);

      cells = new UniformDataArray(typeof(Triangle), 1);

      cells[0] = new Triangle(6, 8, 7);

      unstructuredGrid.AddCells("Triangle", cells);


      cells = new UniformDataArray(typeof(TriangleFan), 1);

      cells[0] = new TriangleFan(new int[] { 12, 13, 14, 15, 16, 17 });

      unstructuredGrid.AddCells("TriangleFan", cells);

      cells = new UniformDataArray(typeof(Tetrahedron), 1);

      cells[0] = new Tetrahedron(18, 19, 20, 21);

      unstructuredGrid.AddCells("Tetrahedron", cells);

      cells = new UniformDataArray(typeof(Line), 4);

      cells[0] = new Line(22, 23);
      cells[1] = new Line(23, 24);
      cells[2] = new Line(24, 25);
      cells[3] = new Line(25, 26);

      //cells[4] = new Line(48, 49);
      //cells[5] = new Line(50, 51);
      //cells[6] = new Line(52, 53);

      unstructuredGrid.AddCells("Line", cells);

      cells = new UniformDataArray(typeof(PolyLine), 1);

      cells[0] = new PolyLine(new int[] { 27, 28, 29, 30, 31 });

      unstructuredGrid.AddCells("PolyLine", cells);

      cells = new UniformDataArray(typeof(Slice), 2);

      Slice s = new Slice();
      s.AddPolyLine(new PolyLine(new int[] { 32, 33, 34, 35, 32 }));
      s.AddPolyLine(new PolyLine(new int[] { 36, 37, 38, 39, 36 }));

      cells[0] = s;

      s = new Slice();
      s.AddPolyLine(new PolyLine(new int[] { 40, 41, 42, 43, 40 }));
      s.AddPolyLine(new PolyLine(new int[] { 44, 45, 46, 47, 44 }));

      cells[1] = s;

      unstructuredGrid.AddCells("Slice", cells);
    }
  }
}
