using System;
using System.Collections.Generic;
using System.Text;
using Zcu.Mve.Core;
using Zcu.Mve.Visualization;
using Zcu.Mve.Numerics;

using System.IO;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;

using System.Xml;
using System.Diagnostics;
using System.Drawing.Design;

namespace Zcu.Mve.UnstrGridRendering
{
    
    /// <summary>
    /// just encapsulate directX texture
    /// </summary>
    [DataObject("Defines texture")]
    class TextureObject : DataObject
    {
        public Texture texture;
        protected String path;
        protected Device _device;

        public TextureObject()
        {
            this.path = null;
        }

        /// <summary>
        /// load texture from given location
        /// </summary>
        /// <param name="path"></param>
        public TextureObject(String path)
        {
            this.path = path;
        }

        /// <summary>
        /// create texture -  called when device is created
        /// </summary>
        /// <param name="device"></param>
        public virtual void Update(Device device)
        {
            _device = device;


            if (path != null)
            {
                try
                {
                    texture = TextureLoader.FromFile(device, path, 0, 0, 0, Usage.None, Format.Unknown, Pool.Managed, Filter.Linear, Filter.Linear, 0);
                }
                catch
                {
                    Debug.WriteLine("texture file: " + path + " failed to open");
                    texture = null;
                    path = "";
                }
            }
            else texture = null;
        }

        /// <summary>
        /// set texture as active (mesh has to have text. coordinates)
        /// </summary>
        public void SetTexture(Device device, SceneObject sceneObject)
        {
            // texture is used only ifif current mesh has text. coordinates
            if (sceneObject.PrefferedVertexAttributes == VertexAttributes.Normal3DTextureCoord ||
                 sceneObject.PrefferedVertexAttributes == VertexAttributes.ColorRGBATextureCoord ||
                 sceneObject.PrefferedVertexAttributes == VertexAttributes.TextureCoord)
            {
                device.SetSamplerState(0, SamplerStageStates.MinFilter, (int)TextureFilter.Linear);
                device.SetSamplerState(0, SamplerStageStates.MagFilter, (int)TextureFilter.Linear);
                device.SetSamplerState(0, SamplerStageStates.AddressU, (int)TextureAddress.Border);
                device.SetSamplerState(0, SamplerStageStates.AddressV, (int)TextureAddress.Border);
                
                device.SetTexture(0, this.texture);
            }
            else device.SetTexture(0, null);
        }

        #region properties
        /// <summary>
        /// texture path and filename
        /// </summary>
        public String Path
        {
            get { return path; }
            set { path = value; }
        }

        /// <summary>
        /// directX texture property
        /// </summary>
        public Texture Texture
        {
            get { return texture; }
            set { texture = value; }
        }
        #endregion

        public override bool CheckConsistence()
        {
            return true;
        }

        /// <summary>
        /// only refence is returned!!
        /// </summary>
        public override IDataObject DeepCopy()
        {
            TextureObject to = new TextureObject(path);
            //DODELAT DEEP COPY
            to.texture = texture;
            return to;
        }

        public override void ReadData(XmlTextReader xmlTextReader)
        {
            //nothing to read - yet
        }

        public override void WriteData(XmlTextWriter xmlTextWriter)
        {
            //nothing to write - yet
        }
    }
}
