using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using Zcu.Mve.Core;
using System.ComponentModel;
using System.Drawing.Design;
using Zcu.Mve.Visualization;
using System.Globalization;
using System.Diagnostics;

using dx3d = Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;


using System.IO;

namespace Zcu.Mve.UnstrGridRendering
{
    /// <summary>
    /// Source of DirectX texture. Texture can be applied on scene object.
    /// </summary>
    [ModuleInfo("Vojtech Hladik", "Source of Material. Texture can be applied on scene object.", IconName = "MaterialModule.ico")]
    [PortInfo("outTexture", "Loaded Texture.")]
    class TextureSource : Module
    {
        private bool dataChanged = true;
        private String path;
        private TextureObject texture;

        #region Properties
        /// <summary>
        /// texture location
        /// </summary>
        /// 

        [Category("Texture")]
        [Description("texture location")]
        public String FileName
        {
            get
            {
                return path;
            }
            set
            {
                path = value;
                dataChanged = true;
            }
        }

        #endregion

        /// <summary>
        /// Constructor, initializes object and adds output port.
        /// </summary>
        public TextureSource()
        {
            AddOutPort("outTexture", typeof(TextureObject));
            
            texture = null;
            path = "";
            texture = new TextureObject(path);
        }


        public override void Execute()
        {
            if (dataChanged == true)
            {
                texture.Path = path;
                SetOutput("outTexture", texture, false);
                dataChanged = false;
            }
            else
            {
                SetOutput("outTexture", texture, true);
            }
        }

         /// <summary>
        /// Writes Xml configuration of current module.
        /// </summary>
        /// <param name="config">XmlElement from map config file which belongs to current module.</param>
        public override void WriteConfig(XmlElement config)
        {
          XmlElement xmlProperties = config.OwnerDocument.CreateElement("Properties");
         
          XmlElement xmlFilename = config.OwnerDocument.CreateElement("texturePath");
          xmlFilename.SetAttribute("type",  path.GetType().FullName);
          xmlFilename.InnerText = path;
          xmlProperties.AppendChild(xmlFilename);

          config.AppendChild(xmlProperties);
        }

        /// <summary>
        /// Reads Xml configuration of current module.
        /// </summary>
        /// <param name="config">XmlElement from map config file which belongs to current module.</param>
        public override void ReadConfig(XmlElement config)
        {
            foreach (XmlElement xeConfig in config.ChildNodes)
            {
                if (xeConfig.Name == "Properties")
                {
                    foreach (XmlElement xeProperties in xeConfig.ChildNodes)
                    {
                        switch (xeProperties.Name)
                        {
                            case "texturePath":
                                path = xeProperties.InnerText;
                                break;
                            default:
                                Debug.WriteLine("Unreaded value:" + xeProperties.Name + " - " + xeProperties.InnerText);
                                break;
                        }
                    }
                }
            }
        }

        /// <summary>
        /// Creates new instance of module and copy all necessary member variables.
        /// This method is used by MapEditor in Copy/Paste process.
        /// Without overriding it returns implicit instance of current module.
        /// </summary>
        public override Module DeepCopy()
        {
            TextureSource textureSource = (TextureSource)base.DeepCopy();
            textureSource.texture = (TextureObject)texture.DeepCopy();
            textureSource.path = path;

            return textureSource;
        }
  
    }
}
