using System;
using System.Collections.Generic;
using System.Text;

using Zcu.Mve.Core;
using Zcu.Mve.Visualization;
using System.ComponentModel;

using System.Xml;
using System.Diagnostics;


namespace Zcu.Mve.UnstrGridRendering
{
    [ModuleInfo("Vojetch Hladik", "Module sends number of frame to play according to given FPS")]
    [PortInfo("frameNumber", "Camera settings.")]
    class Timer : Module
    {
        #region fileds
        private int fps;
        private int numberOfFrames;
        private bool repeat;
        private float frame;
        private long lasttime;
        #endregion

        #region properties
        /// <summary>
        /// animation speed
        /// </summary>
        [DefaultValue(30)]
        public int FPS
        {
            get { return fps; }
            set { fps = value; }
        }
        /// <summary>
        /// number of frames to play (-1 for unknown)
        /// </summary>
        [DefaultValue(1)]
        public int NumberOfFrames
        {
            get { return numberOfFrames; }
            set { numberOfFrames = value; }
        }

        /// <summary>
        /// true if animation should be repeated
        /// </summary>
        [DefaultValue(true)]
        public bool Repeat
        {
            get { return repeat; }
            set { repeat = value; }
        }
        #endregion

        public Timer()
        {
            lasttime = -1;
            frame = 1;

            repeat = true;
            numberOfFrames = 1;
            fps = 30;

            AddOutPort("frameNumber", typeof(Mve.Numerics.Integer), true);
        }

        public override void Execute()
        {
            if (lasttime == -1)
            {
                SetOutput("frameNumber", (Mve.Numerics.Integer)frame);
                lasttime = Environment.TickCount;
            }
            else
            {
                long elapsed = Environment.TickCount - lasttime;
                frame += elapsed / (1000.0f / fps);
                lasttime = Environment.TickCount;
            }

            if (repeat)
                frame %= NumberOfFrames;

            SetOutput("frameNumber", (Mve.Numerics.Integer)frame);

        }

        /// <summary>
        /// Writes Xml configuration of current module.
        /// </summary>
        /// <param name="config">XmlElement from map config file which belongs to current module.</param>
        public override void WriteConfig(XmlElement config)
        {
            XmlElement xmlProperties = config.OwnerDocument.CreateElement("Properties");

            XmlElement xmlFPS = config.OwnerDocument.CreateElement("fps");
            xmlFPS.SetAttribute("type", fps.GetType().FullName);
            xmlFPS.InnerText = fps.ToString();
            xmlProperties.AppendChild(xmlFPS);

            XmlElement xmlNOF = config.OwnerDocument.CreateElement("numberOfFrames");
            xmlNOF.SetAttribute("type", numberOfFrames.GetType().FullName);
            xmlNOF.InnerText = numberOfFrames.ToString();
            xmlProperties.AppendChild(xmlNOF);

            XmlElement xmlRepeat = config.OwnerDocument.CreateElement("repeat");
            xmlRepeat.SetAttribute("type", repeat.GetType().FullName);
            xmlRepeat.InnerText = repeat.ToString();
            xmlProperties.AppendChild(xmlRepeat);

            config.AppendChild(xmlProperties);
        }

        /// <summary>
        /// Reads Xml configuration of current module.
        /// </summary>
        /// <param name="config">XmlElement from map config file which belongs to current module.</param>
        public override void ReadConfig(XmlElement config)
        {
            foreach (XmlElement xeConfig in config.ChildNodes)
            {
                if (xeConfig.Name == "Properties")
                {
                    foreach (XmlElement xeProperties in xeConfig.ChildNodes)
                    {
                        switch (xeProperties.Name)
                        {
                            case "fps":
                                fps = FromString.GetInt(xeProperties.InnerText, -1);
                                break;
                            case "numberOfFrames":
                                numberOfFrames = FromString.GetInt(xeProperties.InnerText, -1);
                                break;
                            case "repeat":
                                repeat = FromString.GetBool(xeProperties.InnerText, true);
                                break;
                            default:
                                Debug.WriteLine("Unreaded value:" + xeProperties.Name + " - " + xeProperties.InnerText);
                                break;
                        }
                    }
                }
            }
        }
    }
}
