using System;
using System.Drawing;
using Zcu.Mve.UnstrGridRendering.Legend;
using System.Xml;
using System.ComponentModel;
using Zcu.Mve.Core;

namespace Zcu.Mve.UnstrGridRendering
{
    [ModuleInfo("Vojtech Hladik", "Legend displaing color table used for volume cut")]
    [PortInfo("inColorMappingFunction", "Class with value to color converting mehods")]
    [PortInfo("outLegend", "Legend to render.")]
    class VolumeCutLegendModule : Mve.Core.Module
    {
        private VolumeCutLegend tl;

        Point _location;
        Size _size;

        [Category("Legend"),
        Description("Legend size in pixels")]
        public Size LegendSize
        {
            get { return _size; }
            set { _size = value; }
        }

        [Category("Legend"),
        Description("Top left corner location")]
        public Point LegendLocation
        {
            get { return _location; }
            set { _location = value; }
        }

        internal Zcu.Mve.UnstrGridRendering.Legend.VolumeCutLegend VolumeCutLegend
        {
            get
            {
                throw new System.NotImplementedException();
            }
            set
            {
            }
        }

        public VolumeCutLegendModule()
        {
            AddOutPort("outLegend", typeof(BaseLegend));
            AddInPort("inColorMappingFunction", typeof(ColorMappingFunction));

            _location = new Point(5, 5);
            _size = new Size(80, 100);

        }

        public override void Execute()
        {
            tl = new VolumeCutLegend(_location, _size);

            tl.MapFunction = (ColorMappingFunction)GetInput("inColorMappingFunction");
            tl.Draw();

            SetOutput("outLegend", (BaseLegend)tl, false);
        }

        /// <summary>
        /// Writes Xml configuration of current module.
        /// </summary>
        /// <param name="config">XmlElement from map config file which belongs to current module.</param>
        public override void WriteConfig(XmlElement config)
        {
            XmlElement xmlProperties = config.OwnerDocument.CreateElement("Properties");

            XmlElement xmlSizeX = config.OwnerDocument.CreateElement("sizeX");
            xmlSizeX.SetAttribute("type", _size.Width.GetType().FullName);
            xmlSizeX.InnerText = _size.Width.ToString();
            xmlProperties.AppendChild(xmlSizeX);

            XmlElement xmlSizeY = config.OwnerDocument.CreateElement("sizeY");
            xmlSizeY.SetAttribute("type", _size.Height.GetType().FullName);
            xmlSizeY.InnerText = _size.Height.ToString();
            xmlProperties.AppendChild(xmlSizeY);

            XmlElement xmllocX = config.OwnerDocument.CreateElement("locationX");
            xmllocX.SetAttribute("type", _location.X.GetType().FullName);
            xmllocX.InnerText = _location.X.ToString();
            xmlProperties.AppendChild(xmllocX);

            XmlElement xmllocY = config.OwnerDocument.CreateElement("locationY");
            xmllocY.SetAttribute("type", _location.Y.GetType().FullName);
            xmllocY.InnerText = _location.Y.ToString();
            xmlProperties.AppendChild(xmllocY);

            config.AppendChild(xmlProperties);
        }

        /// <summary>
        /// Reads Xml configuration of current module.
        /// </summary>
        /// <param name="config">XmlElement from map config file which belongs to current module.</param>
        public override void ReadConfig(System.Xml.XmlElement config)
        {
            foreach (XmlElement xeConfig in config.ChildNodes)
            {
                if (xeConfig.Name == "Properties")
                {
                    foreach (XmlElement xeProperties in xeConfig.ChildNodes)
                    {
                        switch (xeProperties.Name)
                        {
                            case "sizeX":
                                _size.Width = FromString.GetInt(xeProperties.InnerText, 80);
                                break;
                            case "sizeY":
                                _size.Height = FromString.GetInt(xeProperties.InnerText, 100);
                                break;
                            case "locationX":
                                _location.X = FromString.GetInt(xeProperties.InnerText, 5);
                                break;
                            case "locationY":
                                _location.Y = FromString.GetInt(xeProperties.InnerText, 5);
                                break;
                        }
                    }
                }
            }
        }
    }
}
