using System;
using System.Collections.Generic;
using System.Text;
using Zcu.Mve.Core;
using Zcu.Mve.Visualization;
using Zcu.Mve.Numerics;
using System.Xml;

namespace Zcu.Mve.Visualization.Processing
{
    [ModuleInfo("Libor Va", "Generic module for iteration through UniformDataArray. Returns an element at a specified index. Index is specified by the integer port, array is specified by the Array input port. Output potr updates its type upon connecting to output. Checks type compatibility, throws runtime error.")]
    [PortInfo("Index", "Zero-based index of the desired element of array.")]
    [PortInfo("Array", "The input UniformdataArray.")]
    [PortInfo("Out", "Array element at specified index. Updates port type on first connection.")]

    class ArrayIterator:Module
    {
        OutPort outPort;        

        string typeName = "";

        public ArrayIterator()
        {
            AddInPort("Array", typeof(UniformDataArray));
            AddInPort("Index", typeof(Integer));
            AddOutPort("Out", typeof(IDataObject));
            outPort = GetOutPort("Out");            
            this.OutPortDisconnected += new PortDisconnectingEventHandler(ArrayIterator_PortDisconnecting);
            this.OutPortConnecting += new PortConnectingEventHandler(ArrayIterator_OutPortConnecting);
        }

        void ArrayIterator_OutPortConnecting(object source, OutPort portFrom, InPort portTo, out bool allowed)
        {
            if (outPort.LinkList.Count == 0)
            {
                outPort.PortType = portTo.PortType;
                typeName = outPort.PortType.Name;
            }
            allowed = true;
        }

        void ArrayIterator_PortDisconnecting(object source, InPort portTo, out bool allowed)
        {            
            allowed = true;
            if (outPort.LinkList.Count == 0)
                typeName = "";
        }

        void ArrayIterator_PortConnecting(object source, OutPort portFrom, InPort portTo, out bool allowed)
        {
            if (typeName == "")
            {
                outPort.PortType = portTo.PortType;
                typeName = outPort.PortType.FullName;
            }
            allowed = true;
        }

        IDataObject result = null;

        public override void Execute()
        {
            if (IsAllDataSame() && IsAllUserSettingsSame())
            {
                SetOutput("Out", result, true);
            }
            else
            {
                UniformDataArray input = (UniformDataArray)GetInput("Array");
                Integer index = (Integer)GetInput("Index");
                result = input[index];
                SetOutput("Out", result, false);
            }
        }

        public override void WriteConfig(System.Xml.XmlElement config)
        {
            XmlElement typeNameElement = config.OwnerDocument.CreateElement("TypeName");
            typeNameElement.InnerText = outPort.PortType.FullName;
            config.AppendChild(typeNameElement);
        }

        public override void ReadConfig(XmlElement config)
        {
            foreach (XmlElement e in config.ChildNodes)
            {
                if (e.Name == "TypeName")
                {
                    typeName = e.InnerText;
                    if (typeName != "")
                    {
                        try
                        {
                            
                            Type type = Globals.FindType(typeName);
                            if (type != null)
                                outPort.PortType = type;
                        }
                        catch (Exception ex)
                        {
                            WriteDebug("Failed to set output type - possibly missing dll. Specified type: " + typeName);
                        }
                    }
                }
            }
        }
    }
}
