using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;

using Zcu.Mve.Numerics;

namespace Zcu.Mve.Visualization.Loaders
{

    /// <summary>
    /// Holds module constants and enumerations.
    /// </summary>
    public static class BinaryLoaderGlobalConstants
    {

        /// <summary>
        /// Holds allowed grid dimension.
        /// </summary>
        public static readonly int[] allowedDimension = { 2, 3, 4 };

        /// <summary>
        /// Dimension enumeration.
        /// </summary>
        public enum AllowedDimension : int
        {
            /// <summary>
            /// 2D
            /// </summary>
            _2D_ = 2, 
            /// <summary>
            /// 3D
            /// </summary>
            _3D_ = 3,
            /// <summary>
            /// 4D
            /// </summary>
            _4D_ = 4
        }
        
        /// <summary>
        /// Holds names of usable data types.
        /// </summary>
        public static readonly string[] dataTypeNames = {"int8", "uint8", "int16", "uint16", "int32", "uint32",
            "int64", "uint64", "float32", "float64"};
        
        /// <summary>
        /// Data type names enumeration.
        /// </summary>
        public enum DataTypeNames : int
        {
            /// <summary>
            /// sbyte
            /// </summary>
            int8 = 0,
            /// <summary>
            /// byte
            /// </summary>
            uint8 = 1,
            /// <summary>
            /// short
            /// </summary>
            int16 = 2,
            /// <summary>
            /// ushort
            /// </summary>
            uint16 = 3,
            /// <summary>
            /// int
            /// </summary>
            int32 = 4, 
            /// <summary>
            /// uint
            /// </summary>
            uint32 = 5,
            /// <summary>
            /// long
            /// </summary>
            int64 = 6,
            /// <summary>
            ///  ulong
            /// </summary>
            uint64 = 7, 
            /// <summary>
            /// float
            /// </summary>
            float32 = 8,
            /// <summary>
            /// double
            /// </summary>
            float64 = 9
        }

        /// <summary>
        /// Enumeration for specification of cell or point attributes.
        /// </summary>
        public enum PointsOrCells : int
        {
            /// <summary>
            /// point attributes
            /// </summary>
            point = 0, 
            /// <summary>
            /// cell attributes
            /// </summary>
            cell = 1
        }

        /// <summary>
        /// Holds according data type dataSizes in biytes.
        /// </summary>
        public static readonly int[] typesSizesInBytes = { 1, 1, 2, 2, 4, 4, 8, 8, 4, 8 };


        /// <summary>
        /// Regular expression for attribute data type and dimension.
        /// </summary>
        public static readonly Regex dataAttributeRegex;



        /// <summary>
        /// 8 bit constant 0000 0001
        /// </summary>
        public static readonly byte bit8Inverter = 1;

        /// <summary>
        /// 16 bit constant 0000 0000 0000 0001
        /// </summary>
        public static readonly ushort bit16Inverter = 1;

        /// <summary>
        /// 32 bit constant 0000 0000 0000 0000 0000 0000 0000 0001
        /// </summary>
        public static readonly uint bit32Inverter = 1;

        /// <summary>
        /// 64 bit constant 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0001
        /// </summary>
        public static readonly ulong bit64Inverter = 1;

        /// <summary>
        /// Size in bytes of the file reader buffer. Currently 512kB.
        /// </summary>
        public static readonly int readerBufferSize = 512*1024;

        /// <summary>
        /// Static constructor, creates all static instances.
        /// </summary>
        static BinaryLoaderGlobalConstants()
        {
            // Set the data attribute recognition regular expression.
            dataAttributeRegex = new Regex(@"\w+\d*\[\d+\]$");
        }
    }

    /// <summary>
    /// Interface which's impelmentation shall grant acces to Module.SetProgressInfo() method. 
    /// </summary>
    public interface BL_IProgressInfo
    {
        /// <summary>
        /// Sets the progres info of the implementing module.
        /// </summary>
        /// <param name="progress">The set progress.</param>
        /// <param name="stage">The set stage.</param>
        /// <param name="stageCount">The stage count.</param>
        void SetProgressInfo(double progress, int stage, int stageCount);
    }
}
