using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Xml;

namespace Zcu.Mve.Visualization.Processing
{
    /// <summary>
    /// A GUI for EditablePalette which allows creation of own RGB transition function. Stting of multiple controlpoints in color palette and their editting. Y value in palette represents alpha value.
    /// </summary>
    public partial class PaletteGUI : Zcu.Mve.Core.ModuleSetup
    {
        public bod helppoint = null;
        public seznam sez;
        int epsilon = 5; //surrounding of every point for drawing and draging by mouse
        public bool mousedwn = false, FirstStart = true;
        public int function = 0;
        public int sirka;//width of palette

        /// <summary>
        /// Basic setting up of GUI on start.
        /// </summary>
        public PaletteGUI()
        {
            InitializeComponent();

            comboBox1.SelectedIndex = 0;//seting index of combobox to first one

            bod l = new bod(pictureBox1.Width - 2, pictureBox1.Height / 2, Color.Red, null, null);
            bod f = new bod(0, pictureBox1.Height / 2, Color.Blue, null, l);

            l.left = f;

            sez = new seznam(f, l);

            sirka = pictureBox1.Width - 2;

            pictureBox1.Refresh();//refresh
            Repaint();//repaint of GUI
        }

        private void pictureBox1_MouseDown(object sender, MouseEventArgs e)//on mousedown click above GUI
        {

            helppoint = null;

            mousedwn = true;
            helppoint = sez.Select(e.X, e.Y, epsilon);
            Repaint();//repaint of GUI

        }

        private void pictureBox1_MouseUp(object sender, MouseEventArgs e)//on mouse up event
        {
            mousedwn = false;
        }

        private void pictureBox1_MouseMove(object sender, MouseEventArgs e)//method for moving of CP
        {
            int x, y;
            if (mousedwn)
            {
                if (helppoint.left == null || helppoint.right == null)
                {
                    x = helppoint.x;
                }
                else
                {
                    x = Math.Max(e.X, helppoint.left.x + 1);
                    x = Math.Min(x, helppoint.right.x - 1);
                }
                y = Math.Max(e.Y, 0);
                y = Math.Min(y, pictureBox1.Height - 1);

                helppoint.x = x;
                helppoint.y = y;
                pictureBox1.Refresh();
            }

            Repaint();//repaint of GUI
        }

        /// <summary>
        /// Method for repainting GUI.
        /// </summary>
        private void Repaint()
        {
            bod pom = sez.First;
            Graphics g = pictureBox1.CreateGraphics();//creation of graphics

            while (pom != null)//drawing of color palette for all CP
            {

                if (pom.right != null)//drawing until right CP exists
                {
                    if (comboBox1.SelectedIndex == 0)//painting of linear transiton if chosen in combobox
                    {

                        float pomocna = 1.0f / (pom.right.x - pom.x);

                        for (int i = pom.x; i < pom.right.x; i++)
                        {
                            g.DrawLine(new Pen(Color.FromArgb(255, (int)(pomocna * (pom.right.x - i) * pom.color.R + pomocna * (i - pom.x) * pom.right.color.R), (int)(pomocna * (pom.right.x - i) * pom.color.G + pomocna * (i - pom.x) * pom.right.color.G), (int)(pomocna * (pom.right.x - i) * pom.color.B + pomocna * (i - pom.x) * pom.right.color.B))), i, 0, i, pictureBox1.Height - 1);//calculation of color for linear transition
                        }

                        g.DrawLine(Pens.Black, pom.x, pom.y, pom.right.x, pom.right.y);//drawing of color which belongs on that place
                    }

                    if (comboBox1.SelectedIndex == 1)//painting of cosine transiton if chosen in combobox
                    {

                        double w1, w2, vzd;
                        Color c;
                        vzd = pom.right.x - pom.x;

                        for (int i = pom.x; i < pom.right.x; i++)
                        {

                            w1 = 0.5 + 0.5 * Math.Cos(Math.PI * (i - pom.x) / vzd);
                            w2 = 1 - w1;
                            c = Color.FromArgb((int)(pom.color.R * w1 + pom.right.color.R * w2), (int)(pom.color.G * w1 + pom.right.color.G * w2), (int)(pom.color.B * w1 + pom.right.color.B * w2));//calculation of color for cosine transition
                            g.DrawLine(new Pen(c), i, 0, i, pictureBox1.Height - 1);
                        }
                        g.DrawLine(Pens.Black, pom.x, pom.y, pom.right.x, pom.right.y);//drawing of color which belongs on that place
                    }
                }


                g.DrawEllipse(Pens.Black, pom.x - epsilon, pom.y - epsilon, 2 * epsilon, 2 * epsilon);//drawing of surrounding circle around each point
                g.FillEllipse(new SolidBrush(pom.color), pom.x - epsilon, pom.y - epsilon, 2 * epsilon, 2 * epsilon);


                pom = pom.right;
            }

            FirstStart = false;

        }

        
        /// <summary>
        /// Method for deleting of CP.
        /// </summary>
        private void button1_Click(object sender, EventArgs e)
        {
            if (helppoint != null && !mousedwn)
            {
                if (helppoint.left != null && helppoint.right != null)
                {
                    helppoint.left.right = helppoint.right;
                    helppoint.right.left = helppoint.left;
                    helppoint = null;

                    pictureBox1.Refresh();
                    Repaint();
                }
            }
        }

        /// <summary>
        /// Method for altering of CP color.
        /// </summary>
        private void button2_Click(object sender, EventArgs e)
        {
            if (helppoint != null)
            {
                colorDialog1.ShowDialog();

                helppoint.color = colorDialog1.Color;

                pictureBox1.Refresh();
                Repaint();
            }
        }

        /// <summary>
        /// Saving configuration of CP.
        /// </summary>
        private void button3_Click(object sender, EventArgs e)
        {
            //WriteConfig();
        }

        /// <summary>
        /// Loading configuration of CP.
        /// </summary>
        private void button4_Click(object sender, EventArgs e)
        {
            //ReadConfig();
        }

        
        /// <summary>
        /// Method for chosing of color transtion.
        /// </summary>
        private void comboBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (comboBox1.SelectedIndex == 0)
            {
                function = 0;
            }
            else
            {
                function = 1;
            }
            Repaint();
        }

        /// <summary>
        /// Method for reseting of palettes CP.
        /// </summary>
        private void button3_Click_1(object sender, EventArgs e)
        {
            bod l = new bod(pictureBox1.Width - 2, pictureBox1.Height / 2, Color.Red, null, null);
            bod f = new bod(0, pictureBox1.Height / 2, Color.Blue, null, l);

            l.left = f;

            sez = new seznam(f, l);

            pictureBox1.Refresh();
            Repaint();

        }

        private void button4_Click_1(object sender, EventArgs e)
        {
            saveFileDialog1.ShowDialog();
            if (saveFileDialog1.FileName != null)
            {
                XmlDocument doc = new XmlDocument();

                XmlElement main = doc.CreateElement("Points");
                doc.AppendChild(main);

                bod pom = sez.First;

                while (pom != null)
                {
                    XmlElement ec = doc.CreateElement("Point");
                    ec.InnerText = pom.x + ", " + pom.y + ", " + pom.color.R + ", " + pom.color.G + ", " + pom.color.B;
                    main.AppendChild(ec);
                    pom = pom.right;
                }



                doc.Save(saveFileDialog1.FileName);
            }
        }

        private void button5_Click(object sender, EventArgs e)
        {
            openFileDialog1.ShowDialog();
            if (openFileDialog1.FileName != null)
            {
                XmlTextReader r = new XmlTextReader(openFileDialog1.FileName);

                XmlDocument doc = new XmlDocument();

                doc.Load(openFileDialog1.FileName);

                XmlNodeList c = doc.GetElementsByTagName("Point");

                if (c.Count >= 2)
                {
                    bod b = null, first = null;

                    for (int i = 0; i < c.Count; i++)
                    {
                        XmlNode cn = c[i];

                        string[] line = cn.InnerText.Split(',');

                        bod n = new bod(int.Parse(line[0]), int.Parse(line[1]), Color.FromArgb(int.Parse(line[2]), int.Parse(line[3]), int.Parse(line[4])), null, null);

                        if (i != 0)
                        {
                            n.left = b;
                            b.right = n;
                        }
                        else
                        {
                            first = n;
                        }

                        b = n;
                    }

                    sez = new seznam(first, b);
                }

                r.Close();
            }
        }
    }
}