using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing.Design;
using Zcu.Mve.Visualization;
using System.Drawing;
using System.ComponentModel;

namespace Zcu.Mve.Visualization
{
  /// <summary>
  /// Provides a UITypeEditor for visually picking a color. 
  /// </summary>
  public class ColorRGBAEditor : ColorEditor
  {
    /// <summary>
    /// Edits the ColorRGBA object value using the editor.
    /// EditValue for System.Drawing.Color is used to accomplish this task.  
    /// </summary>
    /// <param name="context">An ITypeDescriptorContext that can be used to gain additional context information.</param>
    /// <param name="provider">An IServiceProvider through which editing services may be obtained.</param>
    /// <param name="value">An instance of the value being edited.</param>
    /// <returns>The new value of the object. If the value of the object has not changed, this should return the same object it was passed.</returns>
    public override object EditValue(ITypeDescriptorContext context, IServiceProvider provider, object value)
    {
      ColorRGBA color = (ColorRGBA) value;
 
      value = Color.FromArgb(color.Alpha, color.Red, color.Green, color.Blue);
      Color c = (Color) base.EditValue(context, provider, value);
 
      return new ColorRGBA(c.R, c.G, c.B, c.A);
    }

    /// <summary>
    /// Paints a representative value of the given object to the provided canvas. 
    /// PaintValue for System.Drawing.Color is used to accomplish this task.
    /// </summary>
    /// <param name="e">What to paint and where to paint it.</param>
    public override void PaintValue(PaintValueEventArgs e)
    {
      if (e.Value is ColorRGBA)
      {
        ColorRGBA color = (ColorRGBA) e.Value;
        Color c = Color.FromArgb(color.Alpha, color.Red, color.Green, color.Blue);

        e = new PaintValueEventArgs(e.Context, c, e.Graphics, e.Bounds);
      }

      base.PaintValue(e);
    }
  }
}
