using System;
using Zcu.Mve.Core;
using System.Collections.Generic;
using System.Collections;
using System.Xml;
using Zcu.Mve.Elementals;

namespace Zcu.Mve.Visualization
{
    /// <summary>
    /// Base class for all composed data types.
    /// </summary>
    [DataObject("Base class for all composed data types.")]
    public abstract class DataSet : Zcu.Mve.Core.DataObject
    {
        /// <summary>
        /// Transformation.
        /// </summary>
        public AbstractTransform Trans;



        /// <summary>
        /// Add 26-07-06 for store additional DataSet properties.
        /// Table attributes.
        /// </summary>
        private Dictionary<string, IDataObject> dataSetAttributes = null;

        /// <summary>
        /// Implicit constructor. Usually for XmlLoader only.
        /// </summary>
        public DataSet()
        {
            UpdateModificationTime();
        } // DataSet()

        #region UserTags...
        /// <summary>User defined tags.</summary>
        private Dictionary<string, IDataObject> userTags = null;

        /// <summary>
        /// Public property that encapsulates the user defined tags
        /// </summary>
        public Dictionary<string, IDataObject> UserTags
        {
            get
            {
                if (this.userTags == null)
                {
                    this.userTags = new Dictionary<string, IDataObject>(100);
                }
                return (this.userTags);
            }
        }

        /// <summary>
        /// User tags serialisation. Is called from the default implementation of the WriteData method.
        /// Can be called explicitly or from the overriden ReadData method using the base() calling.
        /// Writes a UserTags element, which contains a number of UserTag elements, which contain serialisations of the user tags.
        /// </summary>
        /// <param name="writer">Writer to be used</param>
        protected void SerializeUserTags(XmlTextWriter writer)
        {
            if (this.userTags!=null)
            {
                writer.WriteStartElement("UserTags");
                foreach(string key in UserTags.Keys)
                {
                    writer.WriteStartElement("UserTag");                    
                    writer.WriteAttributeString("Key",key);                    
                    IDataObject content = this.userTags[key];
                    XmlSaver.WriteDataObjectXML(writer, content);                    
                    writer.WriteEndElement();
                }
                writer.WriteEndElement();
            }
        }

        /// <summary>
        /// User tags deserialization. Is called from the default implementation of the readXML method.
        /// Can be called explicitly or from the overriden ReadData method using the base() calling.
        /// </summary>
        /// <param name="reader">reader to be used. Should point to "data" elemt that contains the data.</param>
        protected void DeserializeUserTags(XmlTextReader reader)
        {
            reader.WhitespaceHandling = WhitespaceHandling.None;
            reader.Read();
            this.userTags = null;
            if (reader.Name == "UserTags")
            {
                this.userTags = new Dictionary<string, IDataObject>();
                reader.Read();
                while (reader.Name == "UserTag")
                {
                    string key = reader.GetAttribute("Key");
                    reader.Read();
                    if (reader.Name == "dataobject")
                    {
                        ArrayList libraries = new ArrayList();
                        reader.Read();
                        if (reader.Name == "libraries")
                        {
                            
                            reader.Read();
                            while (reader.Name == "lib")
                            {
                                reader.Read();
                                string libName = reader.ReadString().Trim();
								libName = libName.Substring(0, libName.Length - 4);
								libraries.Add(libName);
                                reader.ReadEndElement();
                            }
                            reader.ReadEndElement(); //</libraries>
                        }
                        if (reader.Name == "data")
                        {
                            string dataType = reader.GetAttribute("type");
                            IDataObject tag = XmlLoader.ReadFromDataElement(reader, libraries, dataType);
                            reader.ReadEndElement(); // data
                            this.userTags.Add(key, tag);
                        }
                        reader.ReadEndElement(); // dataObject
                    }
                    reader.ReadEndElement(); // UserTag

                }
            }
        }

        #endregion

        public override void ReadData(XmlTextReader xmlTextReader)
        {
            DeserializeUserTags(xmlTextReader);
        }

        public override void WriteData(XmlTextWriter xmlTextWriter)
        {
            SerializeUserTags(xmlTextWriter);
        }

    } // abstract class DataSet
} // namespace