/*
 *		Jiri Skala		A01216
 *
 *		KIV / GSVD		2004/05
 */
using System;
using Zcu.Mve.Core;

namespace Zcu.Mve.Visualization.Processing
{
	/// <summary>
	/// Auxiliary class that provides static methods for converting strings
	/// to basic data types. Methods are prepared to be called from MVE2 modules.
	/// </summary>
	public class Parser
	{
		/// <summary>
		/// Converts given string to a logical value.
		/// </summary>
		/// <param name="s">String to convert.</param>
		/// <param name="moduleName">Name of module which calls this method.</param>
		/// <returns>Returns logical value.</returns>
		public static bool BoolValue(string s, string moduleName)
		{
			bool result;

			// check string content
			if (s == string.Empty)
				throw new MveException(moduleName + ":\n"
					+ "Parsing error reading module configuration. "
					+ "Empty string is not a logical value. "
					+ "Probably missing XML element attribute or wrong attribute name.");

			try
			{
				// try parse given string
				result = bool.Parse(s);
			}
			catch (FormatException)
			{
				// parsing error
				throw new MveException(moduleName + ":\n"
					+ "Parsing error reading module configuration. \""
					+ s + "\" does not represent a logical value.");
			}

			return result;
		}

		/// <summary>
		/// Converts given string to a double (floating point number) value.
		/// </summary>
		/// <param name="s">String to convert.</param>
		/// <param name="moduleName">Name of module which calls this method.</param>
		/// <returns>Returns double value.</returns>
		public static double DoubleValue(string s, string moduleName)
		{
			double result;

			// check string content
			if (s == string.Empty)
				throw new MveException(moduleName + ":\n"
					+ "Parsing error reading module configuration. "
					+ "Empty string is not a floating point number. "
					+ "Probably missing XML element attribute or wrong attribute name.");

			try
			{
				// try parse given string
				result = double.Parse(s);
			}
			catch (FormatException)
			{
				// parsing error
				throw new MveException(moduleName + ":\n"
					+ "Parsing error reading module configuration. \""
					+ s + "\" does not represent a floating point number.");
			}

			return result;
		}
	}
}