using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Zcu.Mve.Core;

namespace Zcu.Mve.Visualization.Processing
{
    /// <summary>
    /// Gui for Arrow Glyph
    /// </summary>
    public partial class ArrowGui : GlyphGui
    {
        #region Variables Declaration
        // arrow attributes, init to def values used from Arro class
        private int tipResolution = Arrow.DefTipResolution;
        /// <summary>
        /// Resolution of arrow tip
        /// </summary>
        public int TipResolution
        {
            get { return tipResolution; }
        }

        private double arrowLength = Arrow.DefArrowLength;
        /// <summary>
        /// Lenght of the arrow
        /// </summary>
        public double ArrowLength
        {
            get { return arrowLength; }
        }

        private double tipRadius = Arrow.DefTipRadius;
        /// <summary>
        /// Radius of arrow tip
        /// </summary>
        public double TipRadius
        {
            get { return tipRadius; }
        }
        private double tipLength = Arrow.DefTipLength;
        /// <summary>
        /// Relative length of arrow tip expected from 0..1
        /// </summary>
        public double TipLength
        {
            get { return tipLength; }
        }
        private int shaftResolution = Arrow.DefShaftResoluion;
        /// <summary>
        /// Resolution of arrow shaft
        /// </summary>
        public int ShaftResoluion
        {
            get { return shaftResolution; }
        }
        private double shaftRadius = Arrow.DefShaftRadius;
        /// <summary>
        /// Radius of arrow shaft
        /// </summary>
        public double ShaftRadius
        {
            get { return shaftRadius; }
        }
        #endregion
       
        /// <summary>
        /// Default constrructor
        /// </summary>
        public ArrowGui()
        {
            InitializeComponent();
            InitGuiValues();// init values of textboxes
        }

        /// <summary>
        /// Return an array of currnet arrow glyph paramatres
        /// </summary>
        /// <returns>An array of currnet arrow glyph paramatres</returns>
        public override List<string> GetParametrs()
        {
            List<string> paramets = new List<string>();
            paramets.Add(tipResolution.ToString());
            paramets.Add(arrowLength.ToString(Globals.nfi));
            paramets.Add(tipRadius.ToString(Globals.nfi));
            paramets.Add(tipLength.ToString(Globals.nfi));
            paramets.Add(shaftResolution.ToString());
            paramets.Add(shaftRadius.ToString(Globals.nfi));
            return paramets;
        }

        /// <summary>
        /// Sets arrow gui attributes according to values in input array
        /// </summary>
        /// <param name="paramets"> new gui values</param>
        public override void SetParametrs(List<string> paramets)
        {
            tipResolution = Int32.Parse(paramets[0]);
            arrowLength = Double.Parse(paramets[1], Globals.nfi);
            tipRadius = Double.Parse(paramets[2], Globals.nfi);
            tipLength = Double.Parse(paramets[3], Globals.nfi);
            shaftResolution = Int32.Parse(paramets[4], Globals.nfi);
            shaftRadius = Double.Parse(paramets[5], Globals.nfi);
            InitGuiValues();
        }

        /// <summary>
        /// Sets arrow attributes to gui components
        /// </summary>
        private void InitGuiValues()
        {
            tipRadiusTBox.Text = tipRadius.ToString(Globals.nfi);
            tipLengthTBox.Text = tipLength.ToString(Globals.nfi);
            shaftResTBox.Text = shaftResolution.ToString(Globals.nfi);
            shaftRadiusTBox.Text = shaftRadius.ToString(Globals.nfi);
            tipResTBox.Text = tipResolution.ToString(Globals.nfi);
            ArrowLengthTbox.Text = arrowLength.ToString(Globals.nfi);
        }


        private void TipRadiusTBox_LostFocus(object sender, EventArgs e)
        {
            bool succeeded = ParseDouble(ref tipRadius, tipRadiusTBox.Text, "Height must be positive double", false);
            if (!succeeded)
            {
                tipRadiusTBox.Text = Arrow.DefTipRadius.ToString();
            }
            else
            {
                glyphParametrsChanged = true;
            }
        }

        private void TipRadiusTBar_Scroll(object sender, EventArgs e)
        {
            tipRadiusTBox.Text = tipRadiusTBar.Value.ToString();
            TipRadiusTBox_LostFocus(null, null);
        }

        private void TipLengthTBox_LostFocus(object sender, EventArgs e)
        {
            bool succeeded = ParseDouble(ref tipLength, tipLengthTBox.Text, "Height must be positive double", false);
            if (!succeeded)
            {
                tipLengthTBox.Text = Arrow.DefTipLength.ToString();
            }
            else
            {
                glyphParametrsChanged = true;
            }
        }

        private void TipLengthTBar_Scroll(object sender, EventArgs e)
        {
            tipLengthTBox.Text = tipLengthTBar.Value.ToString();
            TipLengthTBox_LostFocus(null, null);
        }

        private void ShaftResTBox_LostFocus(object sender, EventArgs e)
        {
            bool succeeded = ParseInt(ref shaftResolution, shaftResTBox.Text, "Coordinates must be positive integers");
            if (!succeeded)
            {
                shaftResTBox.Text = Arrow.DefShaftResoluion.ToString();
            }
            else
            {
                glyphParametrsChanged = true;
            }
        }

        private void ShaftResTBar_Scroll(object sender, EventArgs e)
        {
            shaftResTBox.Text = shaftResTBar.Value.ToString();
            ShaftResTBox_LostFocus(null, null);
        }

        private void ShaftRadiusTBox_LostFocus(object sender, EventArgs e)
        {
            bool succeeded = ParseDouble(ref shaftRadius, shaftRadiusTBox.Text, "Height must be positive double", false);
            if (!succeeded)
            {
                shaftRadiusTBox.Text = Arrow.DefTipRadius.ToString();
            }
            else
            {
                glyphParametrsChanged = true;
            }
        }

        private void ShafRadiusTBar_Scroll(object sender, EventArgs e)
        {
            shaftRadiusTBox.Text = shafRadiusTBar.Value.ToString();
            ShaftRadiusTBox_LostFocus(null, null);
        }

        private void TipResTBox_LostFocus(object sender, EventArgs e)
        {
            bool succeeded = ParseInt(ref tipResolution, tipResTBox.Text, "Coordinates must be positive integers");
            if (!succeeded)
            {
                tipResTBox.Text = Arrow.DefTipResolution.ToString();
            }
            else
            {
                glyphParametrsChanged = true;
            }
        }

        private void TipResTBar_Scroll(object sender, EventArgs e)
        {
            tipResTBox.Text = tipResTBar.Value.ToString();
            TipResTBox_LostFocus(null, null);
        }

        private void arrowLengthTBar_Scroll(object sender, EventArgs e)
        {
            ArrowLengthTbox.Text = arrowLengthTBar.Value.ToString();
            ArrowLengthTbox_LostFocus(null, null);
        }

        private void ArrowLengthTbox_LostFocus(object sender, EventArgs e)
        {
            bool succeeded = ParseDouble(ref arrowLength, ArrowLengthTbox.Text, "Height must be positive double", false);
            if (!succeeded)
            {
                ArrowLengthTbox.Text = Arrow.DefArrowLength.ToString();
            }
            else
            {
                glyphParametrsChanged = true;
            }
        }
    }
}
