using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using Zcu.Mve.Core;
using Zcu.Mve.Visualization;
using System.ComponentModel;
using System.Collections;
using System.Xml;
using Zcu.Mve.Numerics;
using Zcu.Mve.Strings;

namespace Zcu.Mve.Visualization.Processing
{
    /// <summary>
    /// Ancestor for other glyph classes. Contains common attributes, functions and integrity checks.
    /// </summary>
    public abstract class  Glyphs : Zcu.Mve.Core.Module
    {
        #region Variables Declaration

        private bool maskPoints = true;
        /// <summary>
        /// Use max number of glyphs condiditon
        /// </summary>
        [Description("If true maximun nuber of glyphs condiditon will be used")]
        public bool MaskPoints
        {
            get { return maskPoints; }
            set { maskPoints = value; }
        }

        private bool addIndexes = false;
        /// <summary>
        /// If true add orig points indexes ti output
        /// </summary>
        [Description(" If true point attribute that contains index of point from wich glyph was created, will be add to output unstrgid")]
        public bool AddIndexes
        {
            get { return addIndexes; }
            set { addIndexes = value; }
        }
        private bool keepAtributes = false;//
        /// <summary>
        /// if true, original attributes will be add to output glyph points
        /// </summary>
        [Description(" If true, original attribute that was used from glyph generation will be add to output unstrgrid")]
        public bool KeepAtributes
        {
            get { return keepAtributes; }
            set { keepAtributes = value; }
        }

        private double globalScale = 1;// scale of all glyphs
        /// <summary>
        /// Global scale value
        /// </summary>
        [Description(" Value of global scale factor")]
        public double GlobalScale
        {
            get { return globalScale; }
            set { globalScale = value; }
        }
        /// <summary>
        /// Default global scale
        /// </summary>
        public static double DefGlobalScale
        {
            get { return 1.0; }
        }

        private bool useScale = true;// scale glyphs accrodint to their attributes
        /// <summary>
        /// True if local scaleling is used
        /// </summary>
        [Description(" If true local scalin will be perfomed")]
        public bool UseScale
        {
            get { return useScale; }
            set { useScale = value; }
        }

        private bool useGlobalScale = true; // uf true use global scale
        /// <summary>
        /// True if global scaling is used
        /// </summary>
        [Description(" If true global scaling wiil be perfomed")]
        public bool UseGlobalScale
        {
            get { return useGlobalScale; }
            set { useGlobalScale = value; }
        }

        private bool randomMode = true;
        /// <summary>
        /// If true points from which glyphs are created are chosen randomly, else equally
        /// </summary>
        [Description(" if true glzph will be generated in random order, else in equal")]
        public bool RandomMode
        {
            get { return randomMode; }
            set { randomMode = value; }
        }

        private bool orient = true;// 
        /// <summary>
        /// If true, glzphs will be rotated according vetor attributes
        /// </summary>
        [Description(" If true glyphs will be oriented accoeding to vector input attributes")]
        public bool Orient
        {
            get { return orient; }
            set { orient = value; }
        }

        /// <summary>
        /// Actual glyph
        /// </summary>
        protected Glyph3D glyph = new Box(0.01, 0.01, 0.01, new Center(0, 0, 0));// default glyph
        /// <summary>
        /// Actual glyph
        /// </summary>
        public Glyph3D Glyph
        {
            set { glyph = value; }
        }

        private string glyphType = "Box";
        /// <summary>
        /// Type of actual Glyph (box , arrow, cone, sphere, cylinder)
        /// </summary>
        public string GlyphType
        {
            get { return glyphType; }
            set { glyphType = value; }
        }

        private string attributeType = "Vector3D";
        /// <summary>
        /// atribute type (Vecto3D, Normal3D, Scalar, Integer)
        /// </summary>
        [Description(" Type of actual glyph (sphere, cone, cylinder , arrow, box)")]
        public string AttributeType
        {
            get { return attributeType; }
            set { attributeType = value; }
        }

        private int maxGlyphs = 5000;
        /// <summary>
        /// Maximal number of glyphs that will be generated
        /// </summary>
        [Description(" Maximal number of glyphs")]
        public int MaxGlyphs
        {
            set { maxGlyphs = value; }
            get { return maxGlyphs; }
        }

        private String attributeName = "";
        /// <summary>
        /// Name of atribute that will be used for glyph creation
        /// </summary>
        [Description(" Name of attribute that will used to generate glyphs")]
        public String AttributeName
        {
            set { attributeName = value; }
            get { return attributeName; }
        }

        private bool valueChanged = false;
        /// <summary>
        /// variable deteminig if any imput parametr has changed
        /// </summary>
        public bool ValueChanged
        {
            set { valueChanged = value; }
        }

        private string indexAttributeName = "Indexes";
        /// <summary>
        /// Name of attributte that contains orig points indexes
        /// </summary>
        [Description(" Name of attribute that contains original point indexes")]
        public string IndexAttributeName
        {
            set { indexAttributeName = value; }
            get { return indexAttributeName; }
        }

        private string originalAttributeName = "OriginalAtribute";
        /// <summary>
        /// Name of attribute that contains original attribte values
        /// </summary>
        [Description(" Name afo ettribute that contains original attribute values")]
        public string OriginalAttributeName
        {
            get { return originalAttributeName; }
            set { originalAttributeName = value; }
        }

        /// <summary>
        /// Output unstrGrid with glyphs
        /// </summary>
        protected UnstrGrid outputGrid;// output unstrgrid
        private ModulGui gui;// modul gui
        #endregion
        #region Init

        /// <summary>
        /// Constructor initialize modul gui and adds necessary ports
        /// </summary>
        public Glyphs()
        {
            gui = new ModulGui(this);
            gui.SetDefaultValues(null);
            AddPorts();
        }
        /// <summary>
        /// prototype of function that adds port to module
        /// </summary>
        protected abstract void AddPorts();

        /// <summary>
        /// Prototype of function that procces input data and creates glyphs
        /// </summary>
        protected abstract void ProccesInput();
        
        /// <summary>
        /// Writes current module attributes to config
        /// </summary>
        /// <param name="config"> xml node where modul attributes will be saved</param>
        public override void WriteConfig(XmlElement config)
        {
            config.SetAttribute("AttributeType", AttributeType.ToString());
            config.SetAttribute("MaskPoints", MaskPoints.ToString());
            config.SetAttribute("KeepAtributes", KeepAtributes.ToString());
            config.SetAttribute("GlobalScale", GlobalScale.ToString(Globals.nfi));//GlobalScale
            config.SetAttribute("RandomMode", RandomMode.ToString());//RandomMode
            config.SetAttribute("UseScale", UseScale.ToString()); //UseScale
            config.SetAttribute("UseGlobalScale", UseGlobalScale.ToString()); //UseGlobalScale
            config.SetAttribute("Orient", Orient.ToString()); //Orient
            config.SetAttribute("glyphType", glyphType.ToString()); //glyphType
            config.SetAttribute("MaxGlyphs", MaxGlyphs.ToString()); //MaxGlyphs
            config.SetAttribute("attributeName", attributeName.ToString());  //atributeName
            config.SetAttribute("AddIndexes", AddIndexes.ToString());
            config.SetAttribute("indexAttributeName", indexAttributeName);
            config.SetAttribute("OriginalAttributeName", OriginalAttributeName);
            XmlElement child = config.OwnerDocument.CreateElement("glyphAtributes");
            List<string> glyphParametrs = gui.GetGlyphParamatrs();
            for (int i = 0; i < glyphParametrs.Count; i++)
            {
                child.SetAttribute("Attr"+i.ToString(), glyphParametrs[i]);
            }
            config.AppendChild(child);
        }

        /// <summary>
        /// Reads saved modul attributes from config
        /// </summary>
        /// <param name="config"> xml node where modul attributes are saved</param>
        public override void ReadConfig(XmlElement config)
        {
            XmlAttributeCollection attributes = config.Attributes;
            for (int i = 0; i < attributes.Count; i++)
            {
                switch (attributes[i].LocalName)
                {
                    case "AttributeType": AttributeType = (attributes[i].Value);
                        break;
                    case "MaskPoints": MaskPoints = bool.Parse(attributes[i].Value);
                        break;
                    case "KeepAtributes": KeepAtributes = bool.Parse(attributes[i].Value);
                        break;
                    case "GlobalScale": GlobalScale = double.Parse(attributes[i].Value, Globals.nfi);
                        break;
                    case "RandomMode": RandomMode = bool.Parse(attributes[i].Value);
                        break;
                    case "UseScale": UseScale = bool.Parse(attributes[i].Value);
                        break;
                    case "UseGlobalScale": UseGlobalScale = bool.Parse(attributes[i].Value);
                        break;
                    case "Orient": Orient = bool.Parse(attributes[i].Value);
                        break;
                    case "glyphType": glyphType = (attributes[i].Value);
                        break;
                    case "MaxGlyphs": MaxGlyphs = Int32.Parse(attributes[i].Value);
                        break;
                    case "attributeName": attributeName = (attributes[i].Value);
                        break;
                    case "AddIndexes": AddIndexes = bool.Parse(attributes[i].Value);
                        break;
                    case "indexAttributeName": indexAttributeName = attributes[i].Value;
                        break;
                    case "OriginalAttributeName": OriginalAttributeName = attributes[i].Value;
                        break;
                    default:
                        //throw new MveException(this, "Unknown atribute");
                        break;
                }
            }
            // reads saved glyph attributes
            if (config.HasChildNodes && config.FirstChild.LocalName.Equals("glyphAtributes"))
            {
                List<string> glyphAtributes = new List<string>();
                XmlAttributeCollection childAtributes = config.FirstChild.Attributes;
                for (int i = 0; i < childAtributes.Count; i++)
                {
                    glyphAtributes.Add(childAtributes[i].Value);
                }
                gui.SetDefaultValues(glyphAtributes);// set new glyph attributes to gui
            }
            else
            {
                gui.SetDefaultValues(null);// set default glyphAtributes to gui
            }
        }

        /// <summary>
        /// Modul Gui
        /// </summary>
        /// <returns> Modul gui</returns>
        public override ModuleSetup InvokeSetup()
        {
            return gui;
        }

        /// <summary>
        /// Entry point of module.
        /// </summary>
        /// 
        public override void Execute()
        {
            try
            {
                if (!IsAllDataSame() || valueChanged)
                {
                    valueChanged = false;
                    double ticksStart = System.Environment.TickCount;
                    ProccesInput();
                    double ticksEnd = System.Environment.TickCount;
                    Console.WriteLine("Glyphs: " + (ticksEnd - ticksStart) + " ms");
                }
                else
                {
                    SetOutput("UnstrGridOut", outputGrid, true);// set module output when input data didn't changed
                }
            }
            catch (MveException mveEx)
            {
                throw mveEx;
            }
            catch (Exception e)
            {
                throw new MveException(this, "Unhandled exception " + e.Message);
            }
        }

        /// <summary>
        /// Loads user glyph from UnstrGridGlyphIn port if any is available"
        /// </summary>
        protected void GetInputGlyph()
        {
            UnstrGrid inputGlyph = (UnstrGrid)GetInput("UnstrGridGlyphIn");
            if (inputGlyph != null)
            {
                glyph = new Glyph3D();
                for (int i = 0; i < inputGlyph.Points.Count; i++)// read points
                {
                    glyph.GlyphPoints.Add((Point3D)inputGlyph.Points[i]);
                }

                ICollection<string> tmp = inputGlyph.GetCellIds();
                string[] inputCellsIds = new string[tmp.Count];
                tmp.CopyTo(inputCellsIds,0);
                // reads cells, only triangles are allowed, if another cell type is detected an exceptionif thrown
                if(inputCellsIds.Length>1)
                {
                    throw new MveException(this,"Invalid cell types in input glyph, only triangles are allowed");
                }
                UniformDataArray cells = inputGlyph.GetCells(inputCellsIds[0]);
                if ( cells.ElemType.Name.Equals("Triangle"))
                {
                    for (int j = 0; j < cells.Count; j++)
                    {
                        glyph.GlyphCells.Add((Triangle)cells[j]);
                    }
                }
                else
                {
                    throw new MveException(this,"Invalid cell type in input glyph, only triangles are allowed");
                }
            }
        }
        #endregion
        #region AddGlyphs

        /// <summary>
        /// Adds glyph cells to the list of cell for output unstrgird
        /// </summary>
        /// <param name="outputCells"> output list of cells</param>
        /// <param name="numOfGlyphs"> numberf of cells thah should be included to the list</param>
        /// <param name="glyph"> glyph which celll will be used</param>
        protected void AddCells(ref UniformDataArray outputCells, int numOfGlyphs, ref Glyph3D glyph)
        {
            int offset = 0;// offset used for shifting point indexes 
            // determination of number of glyhs
            int maxNumOfglyphs = numOfGlyphs;
            if (MaskPoints)
                maxNumOfglyphs = Math.Min(numOfGlyphs, MaxGlyphs);
            float progressStep = 1 / (float)maxNumOfglyphs;
            float progress = 0;
            int index = 0;
            for (int j = 0; j < maxNumOfglyphs; j++)
            {
                for (int i = 0; i < glyph.GlyphCells.Count; i++)
                {
                    outputCells[index] = new Triangle((glyph.GlyphCells[i]).V1 + offset, (glyph.GlyphCells[i]).V2 + offset, (glyph.GlyphCells[i]).V3 + offset);
                    index++;
                }
                offset += glyph.GlyphPoints.Count;// set offset acording to num of cells already included
                ProgressInfo(progress, 1,2);
                progress += progressStep;
            }
        }

        /// <summary>
        /// Computes rotation matrix, that contains tranformation between normal vector and default glyph direction vector
        /// (0,0,1)
        /// </summary>
        /// <param name="normal"> vetor used for computation of rotation</param>
        /// <returns> transformation between normal vector and default glyph direction vector</returns>
        protected Transform3D AddRotation(Normal3D normal)
        {
            Normal3D z = new Normal3D(0, 1, 0);
            normal.Normalize();
            Normal3D up = normal.CrossProduct(z).CrossProduct(normal);
            up.Normalize();
            Normal3D s = normal.CrossProduct(up);
            s.Normalize();
            Transform3D m = new Transform3D();

            m[0, 0] = (float)s.Nx;
            m[1, 0] = (float)s.Ny;
            m[2, 0] = (float)s.Nz;
            m[3, 0] = 0;

            m[0, 1] = (float)up.Nx;
            m[1, 1] = (float)up.Ny;
            m[2, 1] = (float)up.Nz;
            m[3, 1] = 0;

            m[0, 2] = (float)normal.Nx;
            m[1, 2] = (float)normal.Ny;
            m[2, 2] = (float)normal.Nz;
            m[3, 2] = 0;

            m[3, 0] = 0;
            m[3, 1] = 0;
            m[3, 2] = 0;
            m[3, 3] = 1;
            return m;
        }
        #endregion
    }
}