using System;
using System.Collections.Generic;
using Zcu.Mve.Visualization;
using System.Collections;

namespace Zcu.Mve.Visualization.Processing
{
    /// <summary>
    /// Represents sphere glyph
    /// </summary>
	public class Sphere:Glyph3D
    {
        #region Variables Declaration
        //default sphere attributes values
        private static  int defPhiRes = 5;
        /// <summary>
        /// Default Phi resolution for sheres
        /// </summary>
        public static int DefPhiRes
        {
            get { return defPhiRes; }
        }
        private static int defThetaRes = 3;
        /// <summary>
        /// Default Theta resolution for sheres
        /// </summary>
        public static int DefThetaRes
        {
            get { return defThetaRes; }
        }
        private static double defRadius = 0.01;
        /// <summary>
        /// Default radius for sheres
        /// </summary>
        public static double DefRadius
        {
            get { return defRadius; }
        }

        private static Center defCenter = new Center(0, 0, 0);
        /// <summary>
        /// Default center of the shere
        /// </summary>
        public static Center DefCenter
        {
            get { return defCenter; }
        }
        #endregion

        /// <summary>
        /// Default constructor
        /// </summary>
        /// <param name="phiRes"> sphere phi angle resolution</param>
        /// <param name="thetaRes">sphere phi angle resolution</param>
        /// <param name="radius">sphere radius</param>
        /// <param name="center"> sphere center</param>
        public Sphere(int phiRes, int thetaRes, double radius, Center center)
		{
            int pointCount = phiRes * (thetaRes + 1);
            int triangleCount = 2 * phiRes * thetaRes;

            double phiStep = 2 * Math.PI / phiRes;
            double thetaStep = Math.PI / thetaRes;

            double phi = 0;
            double theta = -Math.PI / 2;

            glyphPoints = new List<Point3D>(pointCount);
            for (int x = 0; x < phiRes; x++)
            {
                theta = -Math.PI / 2;
                for (int y = 0; y < (thetaRes + 1); y++)
                {
                    int index = x * (thetaRes + 1) + y;
                    glyphPoints.Add(new Point3D((radius * Math.Cos(phi) * Math.Cos(theta)) + center.x,
                                                   (radius * Math.Sin(phi) * Math.Cos(theta)) + center.y,
                                                   (-radius * Math.Sin(theta)) + center.z));
                    theta += thetaStep;
                }
                phi += phiStep;
            }

            for (int x = 0; x < (phiRes - 1); x++)
                for (int y = 0; y < thetaRes; y++)
                {
                    int vbindex = x * (thetaRes + 1) + y;
                    glyphCells.Add(new Triangle(vbindex, vbindex + (thetaRes + 1), (vbindex + (thetaRes + 1) + 1)));
                    glyphCells.Add(new Triangle(vbindex, (vbindex + (thetaRes + 1) + 1), (vbindex + 1)));
                }

            for (int y = 0; y < thetaRes; y++)
            {
                int vbindex = (phiRes - 1) * (thetaRes + 1) + y;
                glyphCells.Add(new Triangle((vbindex), (y), (y + 1)));
                glyphCells.Add(new Triangle((vbindex), (y + 1), (vbindex + 1)));
            }
		}
	}
}
