using System;
using System.Collections.Generic;
using System.Text;
using Zcu.Mve.Core;
using Zcu.Mve.Visualization;
using Zcu.Mve.Numerics;
using Zcu.Mve.Strings;
using System.IO;

namespace Zcu.Mve.Visualization.Savers
{
    class ObjSaver:Module
    {

        public ObjSaver()
        {
            AddInPort("Mesh", typeof(TriangleMesh));
            AddInPort("FileName", typeof(MveString), false);
        }

        private string fileName = "mesh.obj";

        public string FileName
        {
            get { return fileName; }
            set { fileName = value; }
        }

        bool writeUnusedVertices = false;

        public bool WriteUnusedVertices
        {
            get { return writeUnusedVertices; }
            set { writeUnusedVertices = value; }
        }

        public override void Execute()
        {
            if (GetInPort("FileName").Link != null)
            {
                this.fileName = ((MveString)GetInput("FileName")).Value;
            }

            TriangleMesh mesh = (TriangleMesh)GetInput("Mesh");

            StreamWriter sw = new StreamWriter(fileName);

            UniformDataArray points = mesh.Points;

            UniformDataArray triangles = mesh.GetCells(typeof(Triangle))[0];

            bool[] used = new bool[mesh.Points.Count];
            foreach (Triangle t in triangles)
            {
                used[t.V1] = true;
                used[t.V2] = true;
                used[t.V3] = true;
            }

            for (int i = 0; i < points.Count; i++)
            {
                if (used[i]||writeUnusedVertices)
                {
                    Point3D p = (Point3D)points[i];
                    sw.WriteLine("v " + p.X.ToString(Globals.nfi) + " "
                        + p.Y.ToString(Globals.nfi) + " " + p.Z.ToString(Globals.nfi));
                }
                
            }
            for (int i = 0; i < triangles.Count; i++)
            {
                Triangle t = (Triangle)triangles[i];
                sw.WriteLine("f " + (t.V1 + 1).ToString() +" "+ (t.V2 + 1).ToString() +" "+ (t.V3 + 1).ToString());
            }
            sw.Close();
        }
    }
}
