using System;
using System.Collections.Generic;
using System.Text;
using Zcu.Mve.Core;
using Zcu.Mve.Visualization;
using System.ComponentModel;

namespace Zcu.Mve.Visualization.Sources
{
    
    [ModuleInfo("Libor Va", "A module that constructs a single PointND of any dimension. Use the Coords property to set the point coordinates.", IconName = "PointNDSource.ico")]
    [PortInfo("Point", "Resulting PointND")]
    class PointNDSource:Module
    {
        string coords = "0,0,0,0";
        double[] coordsValues = { 0, 0, 0, 0 };
        PointND result;
        bool paramsChanged = true;

        /// <summary>
        /// Coordinates of the point. Should be a comma separated list of floating point numbers.
        /// </summary>
        [Description("Coordinates of the point. Should be a comma separated list of floating point numbers.")]
        public string Coords
        {
            get { return coords; }
            set
            {
                try
                {
                    string[] values = value.Split(new char[] { ',' });
                    coordsValues = new double[values.Length];
                    for (int i = 0; i < values.Length; i++)
                    {
                        coordsValues[i] = double.Parse(values[i], Globals.nfi);
                    }
                    coords = value;
                    paramsChanged = true;
                }
                catch (FormatException)
                {
                    throw new MveException(this, "PointND coordinats must be specified as comma-separated list of double values");
                }
            }
        }

        /// <summary>
        /// Default constructor, adds the output port.
        /// </summary>
        public PointNDSource()
        {
            AddOutPort("Point", typeof(PointND));
        }

        /// <summary>
        /// Parses the Coords property and sets the output.
        /// </summary>
        public override void Execute()
        {
            if (paramsChanged)
            {
                result = new PointND(coordsValues);
                SetOutput("Point", result);
                paramsChanged = false;
            }
            else
            {
                SetOutput("Point", result, true);
            }
        }
    }
}
