using System;
using System.Collections;
using System.Xml;
using System.Diagnostics;
using Zcu.Mve.Core;

namespace Zcu.Mve.Visualization
{
	/// <summary>
	/// Object represent ortogonal, regular two dimensional grid.
	/// Structure consists of pixels (cells) and points. 
	/// Point location is given implicitly. 
	/// Point with zero indices have [0,0] cooordinate values.
	/// Point with [height, width] indices have [1,1] cooordinate values.
	/// </summary>
	[DataObject("Object represent ortogonal, regular two dimensional grid. Structure consists of pixels (cells) and points. Point location is given implicitly. Point with zero indices have [0,0] cooordinate values. Point with [height, width] indices have [1,1] cooordinate values.")]
	public class RegGrid2D : Visualization.Grid2D
	{
		/// <summary>
		/// For XmlLoader use only. Use parametric constructor instead.
		/// </summary>
		public RegGrid2D()
		{
			UpdateModificationTime();
		} // RegGrid2()

		/// <summary>
		/// Creates empty grid
		/// </summary>
		/// <param name="height">Number of pixels in height dimension.</param>
		/// <param name="width">Number of pixels in width dimension.</param>
		public RegGrid2D(int height, int width)
			: base(height, width)
		{
			UpdateModificationTime();
		} // RegGrid2D()	

		/// <summary>
		/// Returns x, y coordinates of point with i, j indices.
		/// </summary>
		/// <param name="i">Row index of point.</param>
		/// <param name="j">Column index of point.</param>
		/// <returns>New  instance of point.</returns>
		public override Point2D GetPoint(int i, int j)
		{
			Point2D point = new Point2D();
			point.Y = (double)i / (double)this.Height;
			point.X = (double)j / (double)this.Width;
			return point;
		}

	} // class RegGrid2D
} // namespace