using System;
using System.Text;
using Zcu.Mve.Core;
using Zcu.Mve.Visualization;
using Zcu.Mve.Numerics;

namespace Zcu.Mve.Visualization.Sources
{	
	[ModuleInfo("Libor Va", "A module that represents a source of a uniform scale matrix. The scale coefficient will be read from the Scale factor input.")]
	[PortInfo("Scale factor", "A scalar value taht will be used as the uniform scale factor.")]
	[PortInfo("Transform matrix", "The resulting transform matrix (Transform3D).")]
	/// <summary>
	/// A module that represents a source of a uniform scale matrix. The scale coefficient will be read from the Scale factor input.	
	/// </summary>
	class ScaleTransformSource:Module
	{
		private Transform3D result = null;
		private bool resultSet = false;

		/// <summary>
		/// Default constructor, adds input and output ports.
		/// </summary>
		public ScaleTransformSource()
		{
			AddInPort("Scale factor", typeof(Scalar));
			AddOutPort("Transform matrix", typeof(Transform3D));
		}

		/// <summary>
		/// Creates the transform matrix. If data are unchanged, then the last result is returned immediately.
		/// </summary>
		public override void Execute()
		{
			if ((!IsAllDataSame()) || (!resultSet))
			{
				Scalar factor = (Scalar)GetInput("Scale factor");
				result = new Transform3D();
				result.SetScale(factor, factor, factor);
				SetOutput("Transform matrix", result);
				resultSet = true;
			}
			else
			{
				SetOutput("Transform matrix", result, true);
			}
		}
	}
}
