using System;
using System.Collections.Generic;
using System.Text;
using Zcu.Mve.Core;
using Zcu.Mve.Visualization;

namespace Zcu.Mve.Visualization.Sources
{
    [ModuleInfo("Libor Va", "Source of a TriangleMesh containing vertices and triangles of sphere", IconName = "SphereSource.ico")]
    class SphereSource:Module
    {
        public SphereSource()
        {
            AddOutPort("Sphere mesh", typeof(Visualization.TriangleMesh));
        }

        int phiRes = 32, thetaRes = 16;

        public int ThetaRes
        {
            get { return thetaRes; }
            set { thetaRes = value; }
        }

        public int PhiRes
        {
            get { return phiRes; }
            set { phiRes = value; }
        }

        TriangleMesh output = null;

        public override void Execute()
        {
            if (IsAllUserSettingsSame())
            {
                SetOutput("Sphere mesh", output, true);
            }
            else
            {
                double phiStep = Math.PI * 2 / phiRes;
                double thetaStep = Math.PI / thetaRes;
                int vertexCount = phiRes * (thetaRes - 1) + 2;
                UniformDataArray points = new UniformDataArray(typeof(Point3D), vertexCount);
                double phi = 0;

                int index = 0;
                for (int p = 0; p < phiRes; p++)
                {
                    double theta = -(Math.PI / 2) + thetaStep;
                    for (int t = 0; t < (thetaRes - 1); t++)
                    {
                        Point3D point = new Point3D(Math.Sin(phi) * Math.Cos(theta),
                                                Math.Cos(phi) * Math.Cos(theta),
                                                Math.Sin(theta));
                        points[index] = point;
                        index++;

                        theta += thetaStep;
                    }
                    phi += phiStep;
                }
                Point3D pole1 = new Point3D(0, 0, -1);
                points[index] = pole1;
                index++;
                Point3D pole2 = new Point3D(0, 0, 1);
                points[index] = pole2;

                int triangleCount = 2 * (phiRes * (thetaRes - 1));
                UniformDataArray triangles = new UniformDataArray(typeof(Triangle), triangleCount);
                index = 0;
                for (int p = 0; p < (phiRes); p++)
                    for (int t = 0; t < (thetaRes - 2); t++)
                    {
                        int pos = p * (thetaRes - 1) + t;
                        Triangle t1 = new Triangle(pos, pos + 1, (pos + thetaRes - 1) % (vertexCount - 2));
                        Triangle t2 = new Triangle(pos + 1, (pos + thetaRes) % (vertexCount - 2), (pos + thetaRes - 1) % (vertexCount - 2));
                        triangles[index] = t1;
                        index++;
                        triangles[index] = t2;
                        index++;
                    }
                for (int p = 0; p < (phiRes); p++)
                {
                    Triangle t1 = new Triangle(vertexCount - 2, p * (thetaRes - 1), ((p + 1) * (thetaRes - 1)) % (vertexCount - 2));
                    triangles[index] = t1;
                    index++;
                    Triangle t2 = new Triangle(p * (thetaRes - 1) + (thetaRes - 2), vertexCount - 1, ((p + 1) * (thetaRes - 1) + (thetaRes - 2)) % (vertexCount - 2));
                    triangles[index] = t2;
                    index++;
                }
                output = new TriangleMesh();
                output.Points = points;
                output.AddCells("Triangles", triangles);
                SetOutput("Sphere mesh", output);
            }
        }
    }
}

