﻿using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Diagnostics;
using System.Globalization;

using Zcu.Mve.Core;
using Zcu.Mve.Numerics;
using Zcu.Mve.Visualization;

namespace Zcu.Mve.Visualization.Processing
{
    #region Enums
    /// <summary>
    /// Types of comparing operations
    /// </summary>
    [Description("Types of comparing operations")]
    public enum Sign
    {
        /// <summary>
        /// Compare &amp;alt;
        /// </summary>
        Lower,
        /// <summary>
        /// Compare &amp;gt;
        /// </summary>
        Higher,
        /// <summary>
        /// Compare =
        /// </summary>
        Equal,
        /// <summary>
        /// Compare &amp;alt;=
        /// </summary>
        LowerOrEqual,
        /// <summary>
        /// Compare &amp;gt;=
        /// </summary>
        HigherOrEqual
    }
    #endregion

    /// <summary>
    /// Universal main module for cutting, tresholding, etc.
    /// </summary>    
    [ModuleInfo("Roman Soukal", "Module for cutting, tresholding, etc.", IconName = "VolumeCut.ico", Category = "Zcu.Mve.Gsvd")]
    [PortInfo("Input Grid", "Input RegGridND data")]
    [PortInfo("Output Grid", "Output RegGridND data")]
    public class VolumeCut: Module
    {
        /// <summary>
        /// Default constructor.
        /// </summary>
        public VolumeCut()
        {
            AddInPort("Input Grid", typeof(RegGridND));
            AddOutPort("Output Grid", typeof(RegGridND));
        }

        #region Variables
        private RegGridND data = null;
        private string pointAttributesName = "Values";
        private string cuttingAttributesName = "CuttingValues";
        private string resultAttributesName = "ResultValues";
        private bool paramsChanged = false;
        private double changeValue = 0;
        private double cutValue = 1;
        private Sign typeOfComparing = Sign.Lower;
        #endregion

        /// <summary>
        /// Overrided
        /// </summary> 
        public override void Execute()
        {
            if (!IsAllDataSame() || paramsChanged)
            {
                RegGridND old = (RegGridND)this.GetInput("Input Grid");
                data = old.ShallowCopy();
                //IDataObject val = null;
                //if (!data.UserTags.TryGetValue("Min", out val))
                //{
                //    old.UserTags.TryGetValue("Min", out val);
                //    data.UserTags.Add("Min", val);
                //}
                //if (!data.UserTags.TryGetValue("Max", out val))
                //{
                //    old.UserTags.TryGetValue("Max", out val);
                //    data.UserTags.Add("Max", val);
                //}
                //if (!data.UserTags.TryGetValue("Distortion", out val))
                //{
                //    old.UserTags.TryGetValue("Distortion", out val);
                //    data.UserTags.Add("Distortion", val);
                //}

                MveArray values = data.GetPointAttrs(this.pointAttributesName);
                MveArray cuttingValues = data.GetPointAttrs(this.cuttingAttributesName);
                MveArray resultValues = (MveArray)values.DeepCopy();
                int[] sizes = values.Lengths;
                //Visualization.MveArray a = (Visualization.MveArray)cuttingValues;
               // a.
                Console.WriteLine(cuttingValues.ElemType);
                for (int i = 0; i < sizes[0]; i++)
                {
                    this.ProgressInfo(((float)i / sizes[0]));
                    for (int j = 0; j < sizes[1]; j++)
                        for (int k = 0; k < sizes[2]; k++)
                        {
                            int[] coords = { i, j, k };
                            double value = ((Scalar)cuttingValues.Array.GetValue(coords)).Value;
                            if (values.ElemType.Equals(typeof(Scalar)))
                            {
                                switch (typeOfComparing)
                                {
                                    case Sign.Lower:
                                        if (value < cutValue)
                                            resultValues.Array.SetValue(new Scalar(changeValue), coords);
                                        break;
                                    case Sign.Higher:
                                        if (value > cutValue)
                                            resultValues.Array.SetValue(new Scalar(changeValue), coords);
                                        break;
                                    case Sign.Equal:
                                        if (value == cutValue)
                                            resultValues.Array.SetValue(new Scalar(changeValue), coords);
                                        break;
                                    case Sign.LowerOrEqual:
                                        if (value <= cutValue)
                                            resultValues.Array.SetValue(new Scalar(changeValue), coords);
                                        break;
                                    case Sign.HigherOrEqual:
                                        if (value >= cutValue)
                                            resultValues.Array.SetValue(new Scalar(changeValue), coords);
                                        break;
                                }
                            }
                            else if (values.ElemType.Equals(typeof(Integer)))
                            {
                                
                                switch (typeOfComparing)
                                {
                                    case Sign.Lower:
                                        if (value < cutValue)
                                            resultValues.Array.SetValue(new Integer((int)changeValue), coords);
                                        break;
                                    case Sign.Higher:
                                        if (value > cutValue)
                                            resultValues.Array.SetValue(new Integer((int)changeValue), coords);
                                        break;
                                    case Sign.Equal:
                                        if (value == cutValue)
                                            resultValues.Array.SetValue(new Integer((int)changeValue), coords);
                                        break;
                                    case Sign.LowerOrEqual:
                                        if (value <= cutValue)
                                            resultValues.Array.SetValue(new Integer((int)changeValue), coords);
                                        break;
                                    case Sign.HigherOrEqual:
                                        if (value >= cutValue)
                                            resultValues.Array.SetValue(new Integer((int)changeValue), coords);
                                        break;
                                }
                            }
                            else
                                throw new MveException(this, "Unsupported data type: " + cuttingValues.ElemType);
                        }
                }
                data.AddPointAttrs(this.resultAttributesName, resultValues);
                this.SetOutput("Output Grid", this.data, false);
                paramsChanged = false;
            }
            else
            {
                this.SetOutput("Output Grid", data, true);
            }
        }

        #region Properties
        
        /// <summary>
        /// Set new value for cut. Default set to 1
        /// </summary> 
        [Description("Set new value for cut. Default set to 1")]
        public string CutValue
        {
            set
            {
                try
                {
                    cutValue = double.Parse(value.Trim(), NumberFormatInfo.InvariantInfo);
                    this.paramsChanged = true;
                }
                catch (FormatException)
                {
                    throw new MveException(this, "The number is needed in correct form. " +
                        "Example: -5.2");
                }
            }

            get { return this.cutValue.ToString(NumberFormatInfo.InvariantInfo); }
        }

        /// <summary>
        /// Set new value for cutted voxels. Default set to 0
        /// </summary> 
        [Description("Set new value for cutted voxels. Default set to 0")]
        public string ChangeValue
        {
            set
            {
                try
                {
                    changeValue = double.Parse(value.Trim(), NumberFormatInfo.InvariantInfo);
                    this.paramsChanged = true;
                }
                catch (FormatException)
                {
                    throw new MveException(this, "The number is needed in correct form. " +
                        "Example: -5.2");
                }
            }

            get { return this.changeValue.ToString(NumberFormatInfo.InvariantInfo); }
        }

        /// <summary>
        /// Set name of RegGridND point attribute. Default set to Values.
        /// </summary>
        [Description("Set name of RegGridND point attribute. Default set to \"Values\"")]
        public string PointAttributeName
        {
            set
            {
                if (value == string.Empty)
                {
                    throw new MveException(this, "Attribut property cannot be set to an empty string!");
                }
                else
                {
                    this.pointAttributesName = value;
                    this.paramsChanged = true;
                }
            }

            get { return this.pointAttributesName; }
        }

        /// <summary>
        /// Set name of RegGridND attribute with cutting information. Default set to CuttingValues.
        /// </summary>
        [Description("Set name of RegGridND attribute with cutting information. Default set to\"CuttingValues\"")]
        public string CuttingAttributeName
        {
            set
            {
                if (value == string.Empty)
                {
                    throw new MveException(this, "Attribut property cannot be set to an empty string!");
                }
                else
                {
                    this.cuttingAttributesName = value;
                    this.paramsChanged = true;
                }
            }

            get { return this.cuttingAttributesName; }
        }

        /// <summary>
        /// Set name of RegGridND result attribute after cutting. Default set to ResultValues.
        /// </summary>
        [Description("Set name of RegGridND result attribute after cutting. Default set to \"ResultValues\"")]
        public string ResultAttributeName
        {
            set
            {
                if (value == string.Empty)
                {
                    throw new MveException(this, "Attribut property cannot be set to an empty string!");
                }
                else
                {
                    this.resultAttributesName = value;
                    this.paramsChanged = true;
                }
            }

            get { return this.resultAttributesName; }
        }

        /// <summary>
        /// Set type of comparing operation. Default set to Lower (&amp;alt;).
        /// </summary>
        [Description("Set type of comparing operation (<, >, =, <=, >=). Default set to Lower (<)")]
        public Sign TypeOfComparing
        {
            set
            {
                typeOfComparing = value;
                this.paramsChanged = true;
            }

            get { return typeOfComparing; }
        }
        #endregion
    }
}
