using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;

using Zcu.Mve.Core;
using Zcu.Mve.Numerics;
using Zcu.Mve.Visualization;


namespace Zcu.Mve.Visualization.Processing
{
    /// <summary>
    /// Module extracts iso-surface from input volume data with Marching Tetrahedra algorithm. Generates even more triangles/points than Marching. Cubes algorithm - it would be nice
    /// to use some module for triangle mesh simplification or another suitable post-processing. Module shall be used with 'VolLoader' module, which provides desired input data. 
    /// </summary>
    [ModuleInfo("Ji Zapletal", "Marching Tetrahedra iso-surface extractor.\nModule shall be used with 'VolLoader' module, which provides desired input data.", IconName = "MT6.ico", Category = "Gsvd")]
    [PortInfo("InThreshold", "This value defines iso-surface threshold (in percents).\nNon-required input port.")]
    [PortInfo("InRegGridND", "Input volume data.")]
    [PortInfo("Out", "Output data - triangle mesh.")]
    public class MarchingTetrahedra6 : MarchingTetrahedra
    {
        /// <summary> Checks whether a cube is intersected by searched iso-surface and eventually generates appropriate triangles. </summary>
        protected override void GenerateTriangles(Cube cube, int i, int j, int k)
        {
            // moje 
            /*CalculateTetraIndex(cube.p[1], cube.p[2], cube.p[4], cube.p[6]);
            CalculateTetraIndex(cube.p[0], cube.p[2], cube.p[4], cube.p[7]);
            CalculateTetraIndex(cube.p[0], cube.p[1], cube.p[2], cube.p[4]);
            CalculateTetraIndex(cube.p[2], cube.p[4], cube.p[6], cube.p[7]);
            CalculateTetraIndex(cube.p[0], cube.p[2], cube.p[3], cube.p[7]);
            CalculateTetraIndex(cube.p[1], cube.p[4], cube.p[5], cube.p[6]);//*/

            // P.Bourke http://local.wasp.uwa.edu.au/~pbourke/geometry/polygonise/index.html
            CalculateTetraIndex(cube.p[0], cube.p[2], cube.p[3], cube.p[7]);
            CalculateTetraIndex(cube.p[0], cube.p[2], cube.p[6], cube.p[7]);
            CalculateTetraIndex(cube.p[0], cube.p[4], cube.p[6], cube.p[7]);
            CalculateTetraIndex(cube.p[0], cube.p[6], cube.p[1], cube.p[2]);
            CalculateTetraIndex(cube.p[0], cube.p[6], cube.p[1], cube.p[4]);                        
            CalculateTetraIndex(cube.p[5], cube.p[6], cube.p[1], cube.p[4]);//*/
        } // of GenerateTriangles

    } // of MarchingTetrahedra6 class
} // of namespace Zcu.Gsvd.Extractions