﻿using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.Collections.Generic;
using System.Threading;
using System.Text;
using System.ComponentModel;


using System.Diagnostics;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows.Forms.Design;

using Zcu.Mve.Core;
using Zcu.Mve.Numerics;
using Zcu.Mve.Visualization;

namespace Zcu.Mve.Visualization.Processing
{
    [ModuleInfo("Jiří Zapletal", "This module converts input volume from selected data type to output volume data of selected data type.", IconName = "VolumeDataConvertor.ico", Category = "Gsvd")]
    [PortInfo("InVolDataInt", "Input integer volume data.")]
    [PortInfo("OutVolDataDouble", "Output double volume data.")]
    public class VolumeDataConvertor : Zcu.Mve.Core.Module
    {
        public bool paramsChanged = true;
        public ValueDataType convertFrom = ValueDataType.Integer;
        public ValueDataType convertTo = ValueDataType.Scalar;

        private string pointAttributesName = "Values";
        private RegGridND data = null; // input data
        private MveArray dataMveArray = null;  // volume data

        /// <summary>
        /// Constructor. Creates input and output ports.
        /// </summary>
        public VolumeDataConvertor()
        {
            AddInPort("InVolDataInt", typeof(RegGridND));
            AddOutPort("OutVolDataDouble", typeof(RegGridND));
        } // of MarchingCubes

        /// <summary>
        /// Executes module functionality.
        /// </summary>
        public override void Execute()
        {
            ProgressInfo(0f);
            if (!IsAllDataSame() || paramsChanged) // if I have to compute again 
            {
                Convert();
                SetOutput("OutVolDataDouble", data);
            }
            else
            {
                SetOutput("OutVolDataDouble", data, true);
                IsoSurfaceExtractor.GCClean();
            }
        } // of Execute

        public void Convert()
        {
            IsoSurfaceExtractor.GCClean();

            if (GetInput("InVolDataInt") != null)
                data = (RegGridND)GetInput("InVolDataInt");
            
            Console.Write("Converting data...");
            int ticks = Environment.TickCount;

            if (convertFrom == ValueDataType.Integer && convertTo == ValueDataType.Scalar)
                data = IntegerToScalar(data);
            else if (convertFrom == ValueDataType.Scalar && convertTo == ValueDataType.Integer)
                data = ScalarToInteger(data);

            Console.WriteLine("\t\tDone ({0} ms)", Environment.TickCount - ticks);

            ProgressInfo(1f);
            paramsChanged = false;
        }

        private RegGridND IntegerToScalar(RegGridND data)
        {
            MveArray inDataMveArray = data.GetPointAttrs(pointAttributesName);
            int[] size = inDataMveArray.Lengths;
            MveArray outDataMveArray = new MveArray(typeof(Scalar), size);            

            int size2size1 = size[2] * size[1], index1 = 0, index2 = 0;
            double size0 = size[0];
            
            for (int i = 0; i < size[0]; i++)
            {
                index1 = i * size2size1;
                for (int j = 0; j < size[1]; j++)
                {
                    index2 = index1 + j * size[2];
                    for (int k = 0; k < size[2]; k++)
                        outDataMveArray[i, j, k] = (Scalar)(Integer)inDataMveArray[i, j, k];
                }
                ProgressInfo((i + 1) / size0);
            }
            data.RemovePointAttrs(pointAttributesName);
            data.AddPointAttrs(pointAttributesName, outDataMveArray);

            return data;
        }

        private RegGridND ScalarToInteger(RegGridND data)
        {
            MveArray inDataMveArray = data.GetPointAttrs(pointAttributesName);
            int[] size = inDataMveArray.Lengths;
            MveArray outDataMveArray = new MveArray(typeof(Integer), size);

            int size2size1 = size[2] * size[1], index1 = 0, index2 = 0;
            double size0 = size[0];

            for (int i = 0; i < size[0]; i++)
            {
                index1 = i * size2size1;
                for (int j = 0; j < size[1]; j++)
                {
                    index2 = index1 + j * size[2];
                    for (int k = 0; k < size[2]; k++)
                        outDataMveArray[i, j, k] = (Integer)((Scalar)inDataMveArray[i, j, k]).Value;
                }
                ProgressInfo((i + 1) / size0);
            }
            data.RemovePointAttrs(pointAttributesName);
            data.AddPointAttrs(pointAttributesName, outDataMveArray);

            return data;
        }

        #region Properties
        /// <summary>
        /// From what data type to convert.
        /// </summary>
        [Description("From what data type to convert.")]
        public ValueDataType ConvertFrom
        {
            set
            {
                    this.convertFrom = value;
                    this.paramsChanged = true;                
            }

            get { return this.convertFrom; }
        }

        /// <summary>
        /// To what data type to convert.
        /// </summary>
        [Description("To what data type to convert.")]
        public ValueDataType ConvertTo
        {
            set
            {
                this.convertTo = value;
                this.paramsChanged = true;
            }

            get { return this.convertTo; }
        }
        #endregion
    }
}
